@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2010-2011 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2020 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2021 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -51,18 +51,55 @@
 
 #include <winpr/clipboard.h>
 
+/**
+ * FREERDP_CHECK_VERSION:
+ * @major: major version (e.g. 2 for version 2.1.0)
+ * @minor: minor version (e.g. 0 for version 2.1.0)
+ * @release: release version (e.g. 0 for version 2.1.0)
+ *
+ * Returns %TRUE if the version of the FRERDDP header files
+ * is the same as or newer than the passed-in version.
+ *
+ * Returns: %TRUE if FREERDP headers are new enough
+ */
 
-/* Constants to workaround FreeRDP issue #5417 (min resolution in AVC mode) */
-#define AVC_MIN_DESKTOP_WIDTH 642
+#define FREERDP_CHECK_VERSION(major,minor,revision)                                  \
+	(FREERDP_VERSION_MAJOR > (major) ||                                       \
+	 (FREERDP_VERSION_MAJOR == (major) && FREERDP_VERSION_MINOR > (minor)) || \
+	 (FREERDP_VERSION_MAJOR == (major) && FREERDP_VERSION_MINOR == (minor) && \
+	  FREERDP_VERSION_REVISION >= (revision)))
+
+/**
+ * Constants to workaround FreeRDP issue #5417 (min resolution in AVC mode)
+ * Must be 4 aligned (multiple of 4).
+ * We should instead check destRect and be sure the condition is met
+ * See https://gitlab.com/Remmina/Remmina/-/issues/2507 for a deep discussion
+ */
+#define AVC_MIN_DESKTOP_WIDTH 644
 #define AVC_MIN_DESKTOP_HEIGHT 480
 
 typedef struct rf_context rfContext;
 
 #define GET_PLUGIN_DATA(gp) (rfContext *)g_object_get_data(G_OBJECT(gp), "plugin-data")
 
+/* Performance Flags, from freerdp source
+ * PERF_FLAG_NONE 0x00000000
+ * PERF_DISABLE_WALLPAPER 0x00000001
+ * PERF_DISABLE_FULLWINDOWDRAG 0x00000002
+ * PERF_DISABLE_MENUANIMATIONS 0x00000004
+ * PERF_DISABLE_THEMING 0x00000008
+ * PERF_DISABLE_CURSOR_SHADOW 0x00000020
+ * PERF_DISABLE_CURSORSETTINGS 0x00000040
+ * PERF_ENABLE_FONT_SMOOTHING 0x00000080
+ * PERF_ENABLE_DESKTOP_COMPOSITION 0x00000100
+ */
+/* Poor (default) - all disabled */
 #define DEFAULT_QUALITY_0       0x6f
+/* Medium - only THEMING, CURSOR_SHADOW and CURSORSETTINGS enabled, all other disabled */
 #define DEFAULT_QUALITY_1       0x07
+/* Good - WALLPAPER, FONT_SMOOTHING, DESKTOP_COMPOSITION disabled, all other enabled */
 #define DEFAULT_QUALITY_2       0x01
+/* Best - DESKTOP_COMPOSITION disabled, all other enabled */
 #define DEFAULT_QUALITY_9       0x80
 
 extern RemminaPluginService *remmina_plugin_service;
@@ -81,6 +118,11 @@ struct rf_clipboard {
 	pthread_cond_t		transfer_clip_cond;
 	enum  { SCDW_NONE, SCDW_BUSY_WAIT, SCDW_ABORTING } srv_clip_data_wait;
 	gpointer		srv_data;
+
+	UINT32			server_html_format_id;
+
+	/* Stats for clipboard download */
+	struct timeval clientformatdatarequest_tv;
 };
 typedef struct rf_clipboard rfClipboard;
 
@@ -141,11 +183,16 @@ struct remmina_plugin_rdp_event {
 			CLIPRDR_FORMAT_DATA_REQUEST *pFormatDataRequest;
 		} clipboard_formatdatarequest;
 		struct {
+			gint    Flags;
+			gint    Left;
+			gint    Top;
 			gint	width;
 			gint	height;
 			gint	desktopOrientation;
 			gint	desktopScaleFactor;
 			gint	deviceScaleFactor;
+			gint    physicalWidth;
+			gint    physicalHeight;
 		} monitor_layout;
 	};
 };
@@ -156,7 +203,6 @@ typedef enum {
 	REMMINA_RDP_UI_CONNECTED,
 	REMMINA_RDP_UI_RECONNECT_PROGRESS,
 	REMMINA_RDP_UI_CURSOR,
-	REMMINA_RDP_UI_RFX,
 	REMMINA_RDP_UI_NOCODEC,
 	REMMINA_RDP_UI_CLIPBOARD,
 	REMMINA_RDP_UI_EVENT
@@ -165,8 +211,7 @@ typedef enum {
 typedef enum {
 	REMMINA_RDP_UI_CLIPBOARD_FORMATLIST,
 	REMMINA_RDP_UI_CLIPBOARD_GET_DATA,
-	REMMINA_RDP_UI_CLIPBOARD_SET_DATA,
-	REMMINA_RDP_UI_CLIPBOARD_SET_CONTENT
+	REMMINA_RDP_UI_CLIPBOARD_SET_DATA
 } RemminaPluginRdpUiClipboardType;
 
 typedef enum {
@@ -263,11 +308,11 @@ struct rf_context {
 	RDP_PLUGIN_DATA		drdynvc_data[5];
 	gchar			rdpsnd_options[20];
 
-	RFX_CONTEXT *		rfx_context;
 	gboolean		rdpgfxchan;
 
 	gboolean		connected;
 	gboolean		is_reconnecting;
+	gboolean		stop_reconnecting_requested;
 	/* orphaned: rf_context has still one or more libfreerdp thread active,
 	 * but no longer maintained by an open RemminaProtocolWidget/tab.
 	 * When the orphaned thread terminates, we must cleanup rf_context.
