@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2010-2011 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2020 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2021 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -51,7 +51,7 @@
 #define GET_PLUGIN_DATA(gp) (RemminaPluginVncData *)g_object_get_data(G_OBJECT(gp), "plugin-data")
 
 static RemminaPluginService *remmina_plugin_service = NULL;
-#define REMMINA_PLUGIN_DEBUG(fmt, ...) remmina_plugin_service->_remmina_debug(__func__, fmt, ##__VA_ARGS__)
+#define REMMINA_PLUGIN_DEBUG(fmt, ...) remmina_plugin_service->_remmina_debug(__func__, fmt, ## __VA_ARGS__)
 
 static int dot_cursor_x_hot = 2;
 static int dot_cursor_y_hot = 2;
@@ -97,6 +97,7 @@ static void onMainThread_cleanup_handler(gpointer data)
 {
 	TRACE_CALL(__func__);
 	struct onMainThread_cb_data *d = data;
+
 	d->cancelled = TRUE;
 }
 
@@ -118,14 +119,15 @@ static void onMainThread_schedule_callback_and_wait(struct onMainThread_cb_data
 }
 
 /**
-   Function check_for_endianness() returns 1, if architecture
-   is little endian, 0 in case of big endian.
+ * Function check_for_endianness() returns 1, if architecture
+ * is little endian, 0 in case of big endian.
  */
 static gboolean check_for_endianness()
 {
-  unsigned int x = 1;
-  char *c = (char*) &x;
-  return (int)*c;
+	unsigned int x = 1;
+	char *c = (char *)&x;
+
+	return (int)*c;
 }
 
 static void remmina_plugin_vnc_event_push(RemminaProtocolWidget *gp, gint event_type, gpointer p1, gpointer p2, gpointer p3)
@@ -333,7 +335,7 @@ static void remmina_plugin_vnc_process_vnc_event(RemminaProtocolWidget *gp)
 				break;
 			case REMMINA_PLUGIN_VNC_EVENT_POINTER:
 				SendPointerEvent(cl, event->event_data.pointer.x, event->event_data.pointer.y,
-						event->event_data.pointer.button_mask);
+						 event->event_data.pointer.button_mask);
 				break;
 			case REMMINA_PLUGIN_VNC_EVENT_CUTTEXT:
 				if (event->event_data.text.text) {
@@ -399,12 +401,13 @@ static void remmina_plugin_vnc_update_quality(rfbClient *cl, gint quality)
 		break;
 	case 0:
 	default:
+		// bpp8 and tight encoding is not supported in libvnc
 		cl->appData.useBGR233 = 1;
-		cl->appData.encodingsString = "tight zrle ultra copyrect hextile zlib corre rre raw";
-		cl->appData.compressLevel = 9;
+		cl->appData.encodingsString = "copyrect zrle ultra zlib hextile corre rre raw";
 		cl->appData.qualityLevel = 1;
 		break;
 	}
+	REMMINA_PLUGIN_DEBUG("Quality: %d", quality);
 }
 
 static void remmina_plugin_vnc_update_colordepth(rfbClient *cl, gint colordepth)
@@ -451,17 +454,17 @@ static void remmina_plugin_vnc_update_colordepth(rfbClient *cl, gint colordepth)
 		break;
 	}
 
-	rfbClientLog ("colordepth          = %d\n", colordepth);
-	rfbClientLog ("format.depth        = %d\n", cl->format.depth);
-	rfbClientLog ("format.bitsPerPixel = %d\n", cl->format.bitsPerPixel);
-	rfbClientLog ("format.blueShift    = %d\n", cl->format.blueShift);
-	rfbClientLog ("format.redShift     = %d\n", cl->format.redShift);
-	rfbClientLog ("format.trueColour   = %d\n", cl->format.trueColour);
-	rfbClientLog ("format.greenShift   = %d\n", cl->format.greenShift);
-	rfbClientLog ("format.blueMax      = %d\n", cl->format.blueMax);
-	rfbClientLog ("format.redMax       = %d\n", cl->format.redMax);
-	rfbClientLog ("format.greenMax     = %d\n", cl->format.greenMax);
-	rfbClientLog ("format.bigEndian    = %d\n", cl->format.bigEndian);
+	rfbClientLog("colordepth          = %d\n", colordepth);
+	rfbClientLog("format.depth        = %d\n", cl->format.depth);
+	rfbClientLog("format.bitsPerPixel = %d\n", cl->format.bitsPerPixel);
+	rfbClientLog("format.blueShift    = %d\n", cl->format.blueShift);
+	rfbClientLog("format.redShift     = %d\n", cl->format.redShift);
+	rfbClientLog("format.trueColour   = %d\n", cl->format.trueColour);
+	rfbClientLog("format.greenShift   = %d\n", cl->format.greenShift);
+	rfbClientLog("format.blueMax      = %d\n", cl->format.blueMax);
+	rfbClientLog("format.redMax       = %d\n", cl->format.redMax);
+	rfbClientLog("format.greenMax     = %d\n", cl->format.greenMax);
+	rfbClientLog("format.bigEndian    = %d\n", cl->format.bigEndian);
 }
 
 static rfbBool remmina_plugin_vnc_rfb_allocfb(rfbClient *cl)
@@ -517,6 +520,7 @@ static gint remmina_plugin_vnc_bits(gint n)
 {
 	TRACE_CALL(__func__);
 	gint b = 0;
+
 	while (n) {
 		b++;
 		n >>= 1;
@@ -583,6 +587,7 @@ static void remmina_plugin_vnc_rfb_fill_buffer(rfbClient *cl, guchar *dest, gint
 	gint rs, gs, bs, rm, gm, bm, rl, gl, bl, rr, gr, br;
 	gint r;
 	guint32 *destptr;
+
 	union {
 		struct {
 			guchar a, r, g, b;
@@ -686,6 +691,19 @@ static void remmina_plugin_vnc_rfb_updatefb(rfbClient *cl, int x, int y, int w,
 	remmina_plugin_vnc_queue_draw_area(gp, x, y, w, h);
 }
 
+static void remmina_plugin_vnc_rfb_finished(rfbClient *cl) __attribute__ ((unused));
+static void remmina_plugin_vnc_rfb_finished(rfbClient *cl)
+{
+	TRACE_CALL(__func__);
+	REMMINA_PLUGIN_DEBUG("FinishedFrameBufferUpdate");
+}
+
+static void remmina_plugin_vnc_rfb_led_state(rfbClient *cl, int value, int pad)
+{
+	TRACE_CALL(__func__);
+	REMMINA_PLUGIN_DEBUG("Led state - value: %d, pad: %d", value, pad);
+}
+
 static gboolean remmina_plugin_vnc_queue_cuttext(RemminaPluginVncCuttextParam *param)
 {
 	TRACE_CALL(__func__);
@@ -708,8 +726,9 @@ static gboolean remmina_plugin_vnc_queue_cuttext(RemminaPluginVncCuttextParam *p
 			text = g_convert_with_fallback(param->text, param->textlen, cur_charset, "ISO-8859-1", "?", &br, &bw, NULL);
 			gtk_clipboard_set_text(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD), text, bw);
 			g_free(text);
-		} else
+		} else {
 			g_date_time_unref(t);
+		}
 	}
 	g_free(param->text);
 	g_free(param);
@@ -749,12 +768,12 @@ remmina_plugin_vnc_rfb_password(rfbClient *cl)
 		gboolean save;
 		disablepasswordstoring = remmina_plugin_service->file_get_int(remminafile, "disablepasswordstoring", FALSE);
 		ret = remmina_plugin_service->protocol_plugin_init_auth(gp,
-			(disablepasswordstoring ? 0 : REMMINA_MESSAGE_PANEL_FLAG_SAVEPASSWORD),
-			_("Enter VNC password"),
-			NULL,
-			remmina_plugin_service->file_get_string(remminafile, "password"),
-			NULL,
-			NULL);
+									(disablepasswordstoring ? 0 : REMMINA_MESSAGE_PANEL_FLAG_SAVEPASSWORD),
+									_("Enter VNC password"),
+									NULL,
+									remmina_plugin_service->file_get_string(remminafile, "password"),
+									NULL,
+									NULL);
 		if (ret != GTK_RESPONSE_OK) {
 			gpdata->connected = FALSE;
 			return NULL;
@@ -802,12 +821,12 @@ remmina_plugin_vnc_rfb_credential(rfbClient *cl, int credentialType)
 
 			disablepasswordstoring = remmina_plugin_service->file_get_int(remminafile, "disablepasswordstoring", FALSE);
 			ret = remmina_plugin_service->protocol_plugin_init_auth(gp,
-				(disablepasswordstoring ? 0 : REMMINA_MESSAGE_PANEL_FLAG_SAVEPASSWORD) | REMMINA_MESSAGE_PANEL_FLAG_USERNAME,
-				_("Enter VNC authentication credentials"),
-				remmina_plugin_service->file_get_string(remminafile, "username"),
-				remmina_plugin_service->file_get_string(remminafile, "password"),
-				NULL,
-				NULL);
+										(disablepasswordstoring ? 0 : REMMINA_MESSAGE_PANEL_FLAG_SAVEPASSWORD) | REMMINA_MESSAGE_PANEL_FLAG_USERNAME,
+										_("Enter VNC authentication credentials"),
+										remmina_plugin_service->file_get_string(remminafile, "username"),
+										remmina_plugin_service->file_get_string(remminafile, "password"),
+										NULL,
+										NULL);
 			if (ret == GTK_RESPONSE_OK) {
 				gboolean save = remmina_plugin_service->protocol_plugin_init_get_savepassword(gp);
 				cred->userCredential.username = remmina_plugin_service->protocol_plugin_init_get_username(gp);
@@ -894,14 +913,22 @@ static void remmina_plugin_vnc_rfb_cursor_shape(rfbClient *cl, int xhot, int yho
 static void remmina_plugin_vnc_rfb_bell(rfbClient *cl)
 {
 	TRACE_CALL(__func__);
+	REMMINA_PLUGIN_DEBUG("Bell message received");
 	RemminaProtocolWidget *gp;
+	RemminaFile *remminafile;
 	GdkWindow *window;
 
 	gp = (RemminaProtocolWidget *)(rfbClientGetClientData(cl, NULL));
+	remminafile = remmina_plugin_service->protocol_plugin_get_file(gp);
+
+	if (remmina_plugin_service->file_get_int(remminafile, "disableserverbell", FALSE))
+		return;
+
 	window = gtk_widget_get_window(GTK_WIDGET(gp));
 
 	if (window)
 		gdk_window_beep(window);
+	REMMINA_PLUGIN_DEBUG("Beep emitted");
 }
 
 /* Translate known VNC messages. It’s for intltool only, not for gcc */
@@ -920,10 +947,11 @@ static void remmina_plugin_vnc_rfb_output(const char *format, ...)
 {
 	TRACE_CALL(__func__);
 	va_list args;
+
 	va_start(args, format);
 	gchar *f, *p, *ff;
 
-	if(!rfbEnableClientLogging)
+	if (!rfbEnableClientLogging)
 		return;
 	/* eliminate the last \n */
 	f = g_strdup(format);
@@ -941,10 +969,12 @@ static void remmina_plugin_vnc_rfb_output(const char *format, ...)
 					 NULL);
 			g_snprintf(vnc_error, MAX_ERROR_LENGTH, ff, p);
 			g_free(ff);
-		} else
+		} else {
 			g_snprintf(vnc_error, MAX_ERROR_LENGTH, _(f), p);
-	} else
+		}
+	} else {
 		g_vsnprintf(vnc_error, MAX_ERROR_LENGTH, _(f), args);
+	}
 	g_free(f);
 	va_end(args);
 
@@ -1145,6 +1175,7 @@ static gboolean remmina_plugin_vnc_main(RemminaProtocolWidget *gp)
 		host = remmina_plugin_service->protocol_plugin_start_direct_tunnel(gp, 5900, TRUE);
 
 		if (host == NULL) {
+			REMMINA_PLUGIN_DEBUG("host is null");
 			gpdata->connected = FALSE;
 			break;
 		}
@@ -1166,17 +1197,34 @@ static gboolean remmina_plugin_vnc_main(RemminaProtocolWidget *gp)
 			cl = rfbGetClient(8, 3, 4);
 			break;
 		}
+		REMMINA_PLUGIN_DEBUG("Color depth: %d", colordepth);
 		cl->MallocFrameBuffer = remmina_plugin_vnc_rfb_allocfb;
 		cl->canHandleNewFBSize = TRUE;
 		cl->GetPassword = remmina_plugin_vnc_rfb_password;
 		cl->GetCredential = remmina_plugin_vnc_rfb_credential;
 		cl->GotFrameBufferUpdate = remmina_plugin_vnc_rfb_updatefb;
+		/**
+		 * @fixme we have to implement FinishedFrameBufferUpdate
+		 * This is to know when the server has finished to send a batch of frame buffer
+		 * updates.
+		 * cl->FinishedFrameBufferUpdate = remmina_plugin_vnc_rfb_finished;
+		 */
+		/**
+		 * @fixme we have to implement HandleKeyboardLedState
+		 * cl->HandleKeyboardLedState = remmina_plugin_vnc_rfb_led_state
+		 */
+		cl->HandleKeyboardLedState = remmina_plugin_vnc_rfb_led_state;
 		cl->GotXCutText = (
 			remmina_plugin_service->file_get_int(remminafile, "disableclipboard", FALSE) ?
 			NULL : remmina_plugin_vnc_rfb_cuttext);
 		cl->GotCursorShape = remmina_plugin_vnc_rfb_cursor_shape;
 		cl->Bell = remmina_plugin_vnc_rfb_bell;
 		cl->HandleTextChat = remmina_plugin_vnc_rfb_chat;
+		/**
+		 * @fixme we have to implement HandleXvpMsg
+		 * cl->HandleXvpMsg = remmina_plugin_vnc_rfb_handle_xvp;
+		 */
+
 		rfbClientSetClientData(cl, NULL, gp);
 
 		if (host[0] == '\0') {
@@ -1204,12 +1252,20 @@ static gboolean remmina_plugin_vnc_main(RemminaProtocolWidget *gp)
 		host = NULL;
 
 		if (remmina_plugin_service->file_get_string(remminafile, "proxy")) {
-			cl->destHost = cl->serverHost;
-			cl->destPort = cl->serverPort;
-			remmina_plugin_service->get_server_port(remmina_plugin_service->file_get_string(remminafile, "proxy"), 5900,
-								&s, &cl->serverPort);
-			cl->serverHost = g_strdup(s);
-			g_free(s);
+			remmina_plugin_service->get_server_port(
+				remmina_plugin_service->file_get_string(remminafile, "server"),
+				5900,
+				&cl->destHost,
+				&cl->destPort);
+			remmina_plugin_service->get_server_port(
+				remmina_plugin_service->file_get_string(remminafile, "proxy"),
+				5900,
+				&cl->serverHost,
+				&cl->serverPort);
+			REMMINA_PLUGIN_DEBUG("cl->serverHost: %s", cl->serverHost);
+			REMMINA_PLUGIN_DEBUG("cl->serverPort: %d", cl->serverPort);
+			REMMINA_PLUGIN_DEBUG("cl->destHost: %s", cl->destHost);
+			REMMINA_PLUGIN_DEBUG("cl->destPort: %d", cl->destPort);
 		}
 
 		cl->appData.useRemoteCursor = (
@@ -1234,11 +1290,16 @@ static gboolean remmina_plugin_vnc_main(RemminaProtocolWidget *gp)
 			vnc_encryption_disable_requested = FALSE;
 		}
 
-		if (rfbInitClient(cl, NULL, NULL))
+		if (rfbInitClient(cl, NULL, NULL)) {
+			REMMINA_PLUGIN_DEBUG("Client initialization successfull");
 			break;
+		} else {
+			REMMINA_PLUGIN_DEBUG("Client initialization failed");
+		}
 
-		/* If the authentication is not called, it has to be a fatel error and must quit */
+		/* If the authentication is not called, it has to be a fatal error and must quit */
 		if (!gpdata->auth_called) {
+			REMMINA_PLUGIN_DEBUG("Client not authenticated");
 			gpdata->connected = FALSE;
 			break;
 		}
@@ -1265,6 +1326,7 @@ static gboolean remmina_plugin_vnc_main(RemminaProtocolWidget *gp)
 	}
 
 	if (!gpdata->connected) {
+		REMMINA_PLUGIN_DEBUG("Client not connected with error: %s", vnc_error);
 		if (cl && !gpdata->auth_called && !(remmina_plugin_service->protocol_plugin_has_error(gp)))
 			remmina_plugin_service->protocol_plugin_set_error(gp, "%s", vnc_error);
 		gpdata->running = FALSE;
@@ -1274,6 +1336,7 @@ static gboolean remmina_plugin_vnc_main(RemminaProtocolWidget *gp)
 		return FALSE;
 	}
 
+	REMMINA_PLUGIN_DEBUG("Client connected");
 	remmina_plugin_service->protocol_plugin_init_save_cred(gp);
 
 	gpdata->client = cl;
@@ -1374,6 +1437,19 @@ static gboolean remmina_plugin_vnc_on_button(GtkWidget *widget, GdkEventButton *
 	return TRUE;
 }
 
+static gint delta_to_mask(float delta, float *accum, gint mask_plus, gint mask_minus)
+{
+	*accum += delta;
+	if (*accum >= 1.0) {
+		*accum = 0.0;
+		return mask_plus;
+	} else if (*accum <= -1.0) {
+		*accum = 0.0;
+		return mask_minus;
+	}
+	return 0;
+}
+
 static gboolean remmina_plugin_vnc_on_scroll(GtkWidget *widget, GdkEventScroll *event, RemminaProtocolWidget *gp)
 {
 	TRACE_CALL(__func__);
@@ -1391,26 +1467,26 @@ static gboolean remmina_plugin_vnc_on_scroll(GtkWidget *widget, GdkEventScroll *
 	switch (event->direction) {
 	case GDK_SCROLL_UP:
 		mask = (1 << 3);
+		gpdata->scroll_y_accumulator = 0;
 		break;
 	case GDK_SCROLL_DOWN:
 		mask = (1 << 4);
+		gpdata->scroll_y_accumulator = 0;
 		break;
 	case GDK_SCROLL_LEFT:
 		mask = (1 << 5);
+		gpdata->scroll_x_accumulator = 0;
 		break;
 	case GDK_SCROLL_RIGHT:
 		mask = (1 << 6);
+		gpdata->scroll_x_accumulator = 0;
 		break;
 #if GTK_CHECK_VERSION(3, 4, 0)
 	case GDK_SCROLL_SMOOTH:
-		if (event->delta_y < 0)
-			mask = (1 << 3);
-		if (event->delta_y > 0)
-			mask = (1 << 4);
-		if (event->delta_x < 0)
-			mask = (1 << 5);
-		if (event->delta_x > 0)
-			mask = (1 << 6);
+		/* RFB does not seems to support SMOOTH scroll, so we accumulate GTK delta requested
+		 * up to 1.0 and then send a normal RFB wheel scroll when the accumulator reaches 1.0 */
+		mask = delta_to_mask(event->delta_y, &(gpdata->scroll_y_accumulator), (1 << 4), (1 << 3));
+		mask |= delta_to_mask(event->delta_x, &(gpdata->scroll_x_accumulator), (1 << 6), (1 << 5));
 		if (!mask)
 			return FALSE;
 		break;
@@ -1476,6 +1552,9 @@ static gboolean remmina_plugin_vnc_on_key(GtkWidget *widget, GdkEventKey *event,
 	if (remmina_plugin_service->file_get_int(remminafile, "viewonly", FALSE))
 		return FALSE;
 
+	gpdata->scroll_x_accumulator = 0;
+	gpdata->scroll_y_accumulator = 0;
+
 	/* When sending key release, try first to find out a previously sent keyval
 	 * to workaround bugs like https://bugs.freedesktop.org/show_bug.cgi?id=7430 */
 
@@ -1598,7 +1677,7 @@ static gboolean remmina_plugin_vnc_open_connection(RemminaProtocolWidget *gp)
 
 	if (pthread_create(&gpdata->thread, NULL, remmina_plugin_vnc_main_thread, gp)) {
 		/* I don’t think this will ever happen… */
-		g_print("Failed to initialize pthread. Falling back to non-thread mode…\n");
+		g_print("Could not initialize pthread. Falling back to non-thread mode…\n");
 		g_timeout_add(0, (GSourceFunc)remmina_plugin_vnc_main, gp);
 		gpdata->thread = 0;
 	}
@@ -1649,7 +1728,7 @@ static gboolean remmina_plugin_vnc_close_connection_timeout(RemminaProtocolWidge
 		gpdata->vnc_buffer = NULL;
 	}
 	g_ptr_array_free(gpdata->pressed_keys, TRUE);
-    g_date_time_unref(gpdata->clipboard_timer);
+	g_date_time_unref(gpdata->clipboard_timer);
 	remmina_plugin_vnc_event_free_all(gp);
 	g_queue_free(gpdata->vnc_event_queue);
 	pthread_mutex_destroy(&gpdata->vnc_event_queue_mutex);
@@ -1745,6 +1824,7 @@ static void remmina_plugin_vnc_keystroke(RemminaProtocolWidget *gp, const guint
 {
 	TRACE_CALL(__func__);
 	RemminaPluginVncData *gpdata = GET_PLUGIN_DATA(gp);
+
 	remmina_plugin_service->protocol_plugin_send_keys_signals(gpdata->drawing_area,
 								  keystrokes, keylen, GDK_KEY_PRESS | GDK_KEY_RELEASE);
 	return;
@@ -1793,21 +1873,29 @@ static void remmina_plugin_vnc_init(RemminaProtocolWidget *gp)
 	gpdata = g_new0(RemminaPluginVncData, 1);
 	g_object_set_data_full(G_OBJECT(gp), "plugin-data", gpdata, g_free);
 
+	gboolean disable_smooth_scrolling = FALSE;
+	RemminaFile *remminafile = remmina_plugin_service->protocol_plugin_get_file(gp);
+
+	disable_smooth_scrolling = remmina_plugin_service->file_get_int(remminafile, "disablesmoothscrolling", FALSE);
+	REMMINA_PLUGIN_DEBUG("Disable smooth scrolling is set to %d", disable_smooth_scrolling);
+
 	gpdata->drawing_area = gtk_drawing_area_new();
 	gtk_widget_show(gpdata->drawing_area);
 	gtk_container_add(GTK_CONTAINER(gp), gpdata->drawing_area);
 
 	gtk_widget_add_events(
 		gpdata->drawing_area,
-		GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK 
+		GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK
 		| GDK_BUTTON_RELEASE_MASK | GDK_KEY_PRESS_MASK
-		| GDK_KEY_RELEASE_MASK 
-#if GTK_CHECK_VERSION(3, 4, 0)
-		| GDK_SMOOTH_SCROLL_MASK
-#endif
+		| GDK_KEY_RELEASE_MASK
 		| GDK_SCROLL_MASK);
 	gtk_widget_set_can_focus(gpdata->drawing_area, TRUE);
 
+	if (!disable_smooth_scrolling) {
+		REMMINA_PLUGIN_DEBUG("Adding GDK_SMOOTH_SCROLL_MASK");
+		gtk_widget_add_events(gpdata->drawing_area, GDK_SMOOTH_SCROLL_MASK);
+	}
+
 
 	g_signal_connect(G_OBJECT(gpdata->drawing_area), "draw", G_CALLBACK(remmina_plugin_vnc_on_draw), gp);
 
@@ -1831,42 +1919,38 @@ static void remmina_plugin_vnc_init(RemminaProtocolWidget *gp)
 /* Array of key/value pairs for color depths */
 static gpointer colordepth_list[] =
 {
-	"32", N_("True color (32 bpp)"),
-	"16", N_("High color (16 bpp)"),
-	"8",  N_("256 colors (8 bpp)"),
+	"32", N_("True colour (32 bpp)"),
+	"16", N_("High colour (16 bpp)"),
+	"8",  N_("256 colours (8 bpp)"),
 	NULL
 };
 
 /* Array of key/value pairs for quality selection */
 static gpointer quality_list[] =
 {
-	"9", N_("Best (slowest)"),
 	"2", N_("Good"),
+	"9", N_("Best (slowest)"),
 	"1", N_("Medium"),
 	"0", N_("Poor (fastest)"),
 	NULL
 };
 
-/* Array of RemminaProtocolSetting for basic settings.
- * Each item is composed by:
- * a) RemminaProtocolSettingType for setting type
- * b) Setting name
- * c) Setting description
- * d) Compact disposition
- * e) Values for REMMINA_PROTOCOL_SETTING_TYPE_SELECT or REMMINA_PROTOCOL_SETTING_TYPE_COMBO
- * f) Setting Tooltip
- */
-static const RemminaProtocolSetting remmina_plugin_vnc_basic_settings[] =
-{
-	{ REMMINA_PROTOCOL_SETTING_TYPE_SERVER,	  "server",	NULL,		     FALSE, "_rfb._tcp",     NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	  "proxy",	N_("Repeater"),	     FALSE, NULL,	     NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	  "username",	N_("Username"),	     FALSE, NULL,	     NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD, "password",	N_("User password"), FALSE, NULL,	     NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT,	  "colordepth", N_("Color depth"),   FALSE, colordepth_list, NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT,	  "quality",	N_("Quality"),	     FALSE, quality_list,    NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_KEYMAP,	  "keymap",	NULL,		     FALSE, NULL,	     NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_END,	  NULL,		NULL,		     FALSE, NULL,	     NULL }
-};
+static gchar repeater_tooltip[] =
+	N_("Connect to VNC using a repeater:\n"
+	   "  • The server field must contain the repeater ID, e.g. ID:123456789\n"
+	   "  • The repeater field have to be set to the repeater IP and port, like:\n"
+	   "    10.10.10.12:5901\n"
+	   "  • From the remote VNC server, you will connect to\n"
+	   "    the repeater, e.g. with x11vnc:\n"
+	   "    x11vnc -connect repeater=ID:123456789+10.10.10.12:5500");
+
+static gchar vnciport_tooltip[] =
+	N_("Listening for remote VNC connection:\n"
+	   "  • The \"Listen on port\" field is the port Remmina will listen to,\n"
+	   "    e.g. 8888\n"
+	   "  • From the remote VNC server, you will connect to\n"
+	   "    Remmina, e.g. with x11vnc:\n"
+	   "    x11vnc -display :0 -connect 192.168.1.36:8888");
 
 /* Array of RemminaProtocolSetting for basic settings.
  * Each item is composed by:
@@ -1875,17 +1959,38 @@ static const RemminaProtocolSetting remmina_plugin_vnc_basic_settings[] =
  * c) Setting description
  * d) Compact disposition
  * e) Values for REMMINA_PROTOCOL_SETTING_TYPE_SELECT or REMMINA_PROTOCOL_SETTING_TYPE_COMBO
- * f) Setting Tooltip
+ * f) Setting tooltip
+ * g) Validation data pointer, will be passed to the validation callback method.
+ * h) Validation callback method (Can be NULL. Every entry will be valid then.)
+ *		use following prototype:
+ *		gboolean mysetting_validator_method(gpointer key, gpointer value,
+ *						    gpointer validator_data);
+ *		gpointer key is a gchar* containing the setting's name,
+ *		gpointer value contains the value which should be validated,
+ *		gpointer validator_data contains your passed data.
  */
+static const RemminaProtocolSetting remmina_plugin_vnc_basic_settings[] =
+{
+	{ REMMINA_PROTOCOL_SETTING_TYPE_SERVER,	  "server",	NULL,		     FALSE, "_rfb._tcp",     NULL,	       NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	  "proxy",	N_("Repeater"),	     FALSE, NULL,	     repeater_tooltip, NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	  "username",	N_("Username"),	     FALSE, NULL,	     NULL,	       NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD, "password",	N_("User password"), FALSE, NULL,	     NULL,	       NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT,	  "colordepth", N_("Colour depth"),  FALSE, colordepth_list, NULL,	       NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT,	  "quality",	N_("Quality"),	     FALSE, quality_list,    NULL,	       NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_KEYMAP,	  "keymap",	NULL,		     FALSE, NULL,	     NULL,	       NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_END,	  NULL,		NULL,		     FALSE, NULL,	     NULL,	       NULL, NULL }
+};
+
+// Same as above.
 static const RemminaProtocolSetting remmina_plugin_vnci_basic_settings[] =
 {
-	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	  "listenport", N_("Listen on port"), FALSE, NULL,	      NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	  "username",	N_("Username"),	      FALSE, NULL,	      NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD, "password",	N_("User password"),  FALSE, NULL,	      NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT,	  "colordepth", N_("Color depth"),    FALSE, colordepth_list, NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT,	  "quality",	N_("Quality"),	      FALSE, quality_list,    NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_KEYMAP,	  "keymap",		NULL,		      FALSE, NULL,	      NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_END,	  NULL,		NULL,		      FALSE, NULL,	      NULL }
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	  "listenport", N_("Listen on port"), FALSE, NULL,	      vnciport_tooltip, NULL, NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	  "username",	N_("Username"),	      FALSE, NULL,	      NULL,		NULL, NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD, "password",	N_("User password"),  FALSE, NULL,	      NULL,		NULL, NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT,	  "colordepth", N_("Colour depth"),   FALSE, colordepth_list, NULL,		NULL, NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT,	  "quality",	N_("Quality"),	      FALSE, quality_list,    NULL,		NULL, NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_KEYMAP,	  "keymap",	NULL,		      FALSE, NULL,	      NULL,		NULL, NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_END,	  NULL,		NULL,		      FALSE, NULL,	      NULL,		NULL, NULL}
 };
 
 /* Array of RemminaProtocolSetting for advanced settings.
@@ -1899,13 +2004,15 @@ static const RemminaProtocolSetting remmina_plugin_vnci_basic_settings[] =
  */
 static const RemminaProtocolSetting remmina_plugin_vnc_advanced_settings[] =
 {
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "showcursor",		 N_("Show remote cursor"),	 TRUE,	NULL, NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "viewonly",		 N_("View only"),		 FALSE, NULL, NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disableclipboard",	 N_("Disable clipboard sync"),	 TRUE,	NULL, NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disableencryption",	 N_("Disable encryption"),	 FALSE, NULL, NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disableserverinput",	 N_("Disable server input"),	 TRUE,	NULL, NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disablepasswordstoring", N_("Forget passwords after use"), FALSE, NULL, NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_END,   NULL,			 NULL,				 FALSE, NULL, NULL }
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "showcursor",		 N_("Show remote cursor"),			TRUE,  NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "viewonly",		 N_("View only"),				FALSE, NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disableclipboard",	 N_("Turn off clipboard sync"),			TRUE,  NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disableencryption",	 N_("Turn off encryption"),			FALSE, NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disableserverinput",	 N_("Prevent local interaction on the server"), TRUE,  NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disableserverbell",	 N_("Ignore remote bell messages"),		FALSE, NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disablepasswordstoring", N_("Forget passwords after use"),		TRUE,  NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disablesmoothscrolling", N_("Disable smooth scrolling"),		FALSE, NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_END,   NULL,			 NULL,						FALSE, NULL, NULL }
 };
 
 /* Array for available features.
@@ -1916,13 +2023,13 @@ static const RemminaProtocolFeature remmina_plugin_vnc_features[] =
 	  quality_list },
 	{ REMMINA_PROTOCOL_FEATURE_TYPE_PREF,	 REMMINA_PLUGIN_VNC_FEATURE_PREF_VIEWONLY,	     GINT_TO_POINTER(REMMINA_PROTOCOL_FEATURE_PREF_CHECK), "viewonly",
 	  N_("View only") },
-	{ REMMINA_PROTOCOL_FEATURE_TYPE_PREF,	 REMMINA_PLUGIN_VNC_FEATURE_PREF_DISABLESERVERINPUT, GINT_TO_POINTER(REMMINA_PROTOCOL_FEATURE_PREF_CHECK), "disableserverinput",N_("Disable server input")  },
-	{ REMMINA_PROTOCOL_FEATURE_TYPE_TOOL,	 REMMINA_PLUGIN_VNC_FEATURE_TOOL_REFRESH,	     N_("Refresh"),					   NULL,		NULL			    },
-	{ REMMINA_PROTOCOL_FEATURE_TYPE_TOOL,	 REMMINA_PLUGIN_VNC_FEATURE_TOOL_CHAT,		     N_("Open Chat…"),					   "face-smile",	NULL			    },
-	{ REMMINA_PROTOCOL_FEATURE_TYPE_TOOL,	 REMMINA_PLUGIN_VNC_FEATURE_TOOL_SENDCTRLALTDEL,     N_("Send Ctrl+Alt+Delete"),			   NULL,		NULL			    },
-	{ REMMINA_PROTOCOL_FEATURE_TYPE_SCALE,	 REMMINA_PLUGIN_VNC_FEATURE_SCALE,		     NULL,						   NULL,		NULL			    },
-	{ REMMINA_PROTOCOL_FEATURE_TYPE_UNFOCUS, REMMINA_PLUGIN_VNC_FEATURE_UNFOCUS,		     NULL,						   NULL,		NULL			    },
-	{ REMMINA_PROTOCOL_FEATURE_TYPE_END,	 0,						     NULL,						   NULL,		NULL			    }
+	{ REMMINA_PROTOCOL_FEATURE_TYPE_PREF,	 REMMINA_PLUGIN_VNC_FEATURE_PREF_DISABLESERVERINPUT, GINT_TO_POINTER(REMMINA_PROTOCOL_FEATURE_PREF_CHECK), "disableserverinput",N_("Prevent local interaction on the server")  },
+	{ REMMINA_PROTOCOL_FEATURE_TYPE_TOOL,	 REMMINA_PLUGIN_VNC_FEATURE_TOOL_REFRESH,	     N_("Refresh"),					   NULL,		NULL					       },
+	{ REMMINA_PROTOCOL_FEATURE_TYPE_TOOL,	 REMMINA_PLUGIN_VNC_FEATURE_TOOL_CHAT,		     N_("Open Chat…"),					   "face-smile",	NULL					       },
+	{ REMMINA_PROTOCOL_FEATURE_TYPE_TOOL,	 REMMINA_PLUGIN_VNC_FEATURE_TOOL_SENDCTRLALTDEL,     N_("Send Ctrl+Alt+Delete"),			   NULL,		NULL					       },
+	{ REMMINA_PROTOCOL_FEATURE_TYPE_SCALE,	 REMMINA_PLUGIN_VNC_FEATURE_SCALE,		     NULL,						   NULL,		NULL					       },
+	{ REMMINA_PROTOCOL_FEATURE_TYPE_UNFOCUS, REMMINA_PLUGIN_VNC_FEATURE_UNFOCUS,		     NULL,						   NULL,		NULL					       },
+	{ REMMINA_PROTOCOL_FEATURE_TYPE_END,	 0,						     NULL,						   NULL,		NULL					       }
 };
 
 /* Protocol plugin definition and features */
@@ -1967,7 +2074,9 @@ static RemminaProtocolPlugin remmina_plugin_vnci =
 	remmina_plugin_vnc_query_feature,               // Query for available features
 	remmina_plugin_vnc_call_feature,                // Call a feature
 	remmina_plugin_vnc_keystroke,                   // Send a keystroke
-	NULL                                            // No screenshot support available
+	NULL,                                           // No screenshot support available
+	NULL,                                           // RCW map event
+	NULL                                            // RCW unmap event
 };
 
 G_MODULE_EXPORT gboolean
