@@ -1,6 +1,6 @@
 /*
  * Remmina - The GTK+ Remote Desktop Client
- * Copyright (C) 2016-2019 Antenore Gatta & Giovanni Panozzo
+ * Copyright (C) 2016-2021 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -51,6 +51,7 @@
 #include "www_utils.h"
 #include "www_plugin.h"
 
+#define UNUSED(x) (void)(x)
 
 #define GET_PLUGIN_DATA(gp) (RemminaPluginWWWData *)g_object_get_data(G_OBJECT(gp), "plugin-data")
 
@@ -70,7 +71,7 @@ typedef struct _RemminaPluginWWWData {
 	gboolean			formauthenticated;
 } RemminaPluginWWWData;
 
-static RemminaPluginService *remmina_plugin_service = NULL;
+RemminaPluginService *remmina_plugin_service = NULL;
 
 void remmina_plugin_www_download_started(WebKitWebContext *context,
 					 WebKitDownload *download, RemminaProtocolWidget *gp)
@@ -86,13 +87,13 @@ void remmina_plugin_www_download_started(WebKitWebContext *context,
 void remmina_plugin_www_response_received(WebKitDownload *download, GParamSpec *ps, RemminaProtocolWidget *gp)
 {
 	TRACE_CALL(__func__);
-	g_debug("Download response received");
+	REMMINA_PLUGIN_DEBUG("Download response received");
 }
 
 void remmina_plugin_www_notify_download(WebKitDownload *download, gchar *destination, RemminaProtocolWidget *gp)
 {
 	TRACE_CALL(__func__);
-	g_debug("Download is finished");
+	REMMINA_PLUGIN_DEBUG("Download is finished");
 	const gchar *dest = webkit_download_get_destination(download);
 	www_utils_send_notification("www-plugin-download-completed-id", _("File downloaded"), dest);
 	//download(gp, webkit_download_get_response(download));
@@ -128,7 +129,7 @@ static gboolean remmina_plugin_www_decide_policy_cb(
 void remmina_plugin_www_decide_nav(WebKitPolicyDecision *decision, RemminaProtocolWidget *gp)
 {
 	TRACE_CALL(__func__);
-	g_debug("Policy decision navigation");
+	REMMINA_PLUGIN_DEBUG("Policy decision navigation");
 	const gchar *url = NULL;
 	WebKitNavigationAction *a =
 		webkit_navigation_policy_decision_get_navigation_action(
@@ -136,25 +137,25 @@ void remmina_plugin_www_decide_nav(WebKitPolicyDecision *decision, RemminaProtoc
 
 	switch (webkit_navigation_action_get_navigation_type(a)) {
 	case WEBKIT_NAVIGATION_TYPE_LINK_CLICKED:
-		g_debug("WEBKIT_NAVIGATION_TYPE_LINK_CLICKED");
+		REMMINA_PLUGIN_DEBUG("WEBKIT_NAVIGATION_TYPE_LINK_CLICKED");
 		url = webkit_uri_request_get_uri(
 			webkit_navigation_action_get_request(a));
-		g_debug("url is %s ", url);
+		REMMINA_PLUGIN_DEBUG("url is %s ", url);
 		break;
 	case WEBKIT_NAVIGATION_TYPE_FORM_SUBMITTED:
-		g_debug("WEBKIT_NAVIGATION_TYPE_FORM_SUBMITTED");
+		REMMINA_PLUGIN_DEBUG("WEBKIT_NAVIGATION_TYPE_FORM_SUBMITTED");
 		break;
 	case WEBKIT_NAVIGATION_TYPE_BACK_FORWARD:
-		g_debug("WEBKIT_NAVIGATION_TYPE_BACK_FORWARD");
+		REMMINA_PLUGIN_DEBUG("WEBKIT_NAVIGATION_TYPE_BACK_FORWARD");
 		break;
 	case WEBKIT_NAVIGATION_TYPE_RELOAD:
-		g_debug("WEBKIT_NAVIGATION_TYPE_RELOAD");
+		REMMINA_PLUGIN_DEBUG("WEBKIT_NAVIGATION_TYPE_RELOAD");
 		break;
 	case WEBKIT_NAVIGATION_TYPE_FORM_RESUBMITTED:
-		g_debug("WEBKIT_NAVIGATION_TYPE_FORM_RESUBMITTED");
+		REMMINA_PLUGIN_DEBUG("WEBKIT_NAVIGATION_TYPE_FORM_RESUBMITTED");
 		break;
 	case WEBKIT_NAVIGATION_TYPE_OTHER:
-		g_debug("WEBKIT_NAVIGATION_TYPE_OTHER");
+		REMMINA_PLUGIN_DEBUG("WEBKIT_NAVIGATION_TYPE_OTHER");
 		break;
 	default:
 		/* Do not navigate to links with a "_blank" target (popup) */
@@ -170,10 +171,66 @@ void remmina_plugin_www_decide_nav(WebKitPolicyDecision *decision, RemminaProtoc
 		break;
 	}
 }
+
+void remmina_plugin_www_on_create(WebKitWebView *webview, WebKitNavigationAction *a, RemminaProtocolWidget *gp)
+{
+	TRACE_CALL(__func__);
+	REMMINA_PLUGIN_DEBUG("New web-view");
+
+	const gchar *url = NULL;
+
+	RemminaPluginWWWData *gpdata;
+	gpdata = (RemminaPluginWWWData *)g_object_get_data(G_OBJECT(gp), "plugin-data");
+
+	switch (webkit_navigation_action_get_navigation_type(a)) {
+	case WEBKIT_NAVIGATION_TYPE_LINK_CLICKED:
+		REMMINA_PLUGIN_DEBUG("WEBKIT_NAVIGATION_TYPE_LINK_CLICKED");
+		url = webkit_uri_request_get_uri(
+			webkit_navigation_action_get_request(a));
+		REMMINA_PLUGIN_DEBUG("Downloading url %s ", url);
+		WebKitDownload *d = webkit_web_view_download_uri(gpdata->webview, url);
+		UNUSED(d);
+
+		break;
+	case WEBKIT_NAVIGATION_TYPE_FORM_SUBMITTED:
+		REMMINA_PLUGIN_DEBUG("WEBKIT_NAVIGATION_TYPE_FORM_SUBMITTED");
+		break;
+	case WEBKIT_NAVIGATION_TYPE_BACK_FORWARD:
+		REMMINA_PLUGIN_DEBUG("WEBKIT_NAVIGATION_TYPE_BACK_FORWARD");
+		break;
+	case WEBKIT_NAVIGATION_TYPE_RELOAD:
+		REMMINA_PLUGIN_DEBUG("WEBKIT_NAVIGATION_TYPE_RELOAD");
+		break;
+	case WEBKIT_NAVIGATION_TYPE_FORM_RESUBMITTED:
+		REMMINA_PLUGIN_DEBUG("WEBKIT_NAVIGATION_TYPE_FORM_RESUBMITTED");
+		/* Filter domains here */
+		/* If the value of “mouse-button” is not 0, then the navigation was triggered by a mouse event.
+		 * test for link clicked but no button ? */
+		url = webkit_uri_request_get_uri(
+			webkit_navigation_action_get_request(a));
+		REMMINA_PLUGIN_DEBUG("Trying to open url: %s", url);
+		webkit_web_view_load_uri(gpdata->webview, url);
+		break;
+	case WEBKIT_NAVIGATION_TYPE_OTHER:         /* fallthrough */
+		REMMINA_PLUGIN_DEBUG("WEBKIT_NAVIGATION_TYPE_OTHER");
+		/* Filter domains here */
+		/* If the value of “mouse-button” is not 0, then the navigation was triggered by a mouse event.
+		 * test for link clicked but no button ? */
+		url = webkit_uri_request_get_uri(
+			webkit_navigation_action_get_request(a));
+		REMMINA_PLUGIN_DEBUG("Trying to open url: %s", url);
+		webkit_web_view_load_uri(gpdata->webview, url);
+		break;
+	default:
+		break;
+	}
+	REMMINA_PLUGIN_DEBUG("WEBKIT_NAVIGATION_TYPE is %d", webkit_navigation_action_get_navigation_type(a));
+}
+
 void remmina_plugin_www_decide_newwin(WebKitPolicyDecision *decision, RemminaProtocolWidget *gp)
 {
 	TRACE_CALL(__func__);
-	g_debug("Policy decision new window");
+	REMMINA_PLUGIN_DEBUG("Policy decision new window");
 
 	const gchar *url = NULL;
 
@@ -186,55 +243,55 @@ void remmina_plugin_www_decide_newwin(WebKitPolicyDecision *decision, RemminaPro
 
 	switch (webkit_navigation_action_get_navigation_type(a)) {
 	case WEBKIT_NAVIGATION_TYPE_LINK_CLICKED:
-		g_debug("WEBKIT_NAVIGATION_TYPE_LINK_CLICKED");
+		REMMINA_PLUGIN_DEBUG("WEBKIT_NAVIGATION_TYPE_LINK_CLICKED");
 		url = webkit_uri_request_get_uri(
 			webkit_navigation_action_get_request(a));
-		g_debug("Downloading url %s ", url);
+		REMMINA_PLUGIN_DEBUG("Downloading url %s ", url);
 		WebKitDownload *d = webkit_web_view_download_uri(gpdata->webview, url);
+		UNUSED(d);
 
 
 		break;
 	case WEBKIT_NAVIGATION_TYPE_FORM_SUBMITTED:
-		g_debug("WEBKIT_NAVIGATION_TYPE_FORM_SUBMITTED");
+		REMMINA_PLUGIN_DEBUG("WEBKIT_NAVIGATION_TYPE_FORM_SUBMITTED");
 		break;
 	case WEBKIT_NAVIGATION_TYPE_BACK_FORWARD:
-		g_debug("WEBKIT_NAVIGATION_TYPE_BACK_FORWARD");
+		REMMINA_PLUGIN_DEBUG("WEBKIT_NAVIGATION_TYPE_BACK_FORWARD");
 		break;
 	case WEBKIT_NAVIGATION_TYPE_RELOAD:
-		g_debug("WEBKIT_NAVIGATION_TYPE_RELOAD");
+		REMMINA_PLUGIN_DEBUG("WEBKIT_NAVIGATION_TYPE_RELOAD");
 		break;
 	case WEBKIT_NAVIGATION_TYPE_FORM_RESUBMITTED:
-		g_debug("WEBKIT_NAVIGATION_TYPE_FORM_RESUBMITTED");
+		REMMINA_PLUGIN_DEBUG("WEBKIT_NAVIGATION_TYPE_FORM_RESUBMITTED");
 		/* Filter domains here */
 		/* If the value of “mouse-button” is not 0, then the navigation was triggered by a mouse event.
 		 * test for link clicked but no button ? */
 		url = webkit_uri_request_get_uri(
 			webkit_navigation_action_get_request(a));
-		g_debug("Trying to open url: %s", url);
+		REMMINA_PLUGIN_DEBUG("Trying to open url: %s", url);
 		webkit_web_view_load_uri(gpdata->webview, url);
 		break;
 	case WEBKIT_NAVIGATION_TYPE_OTHER:         /* fallthrough */
-		g_debug("WEBKIT_NAVIGATION_TYPE_OTHER");
+		REMMINA_PLUGIN_DEBUG("WEBKIT_NAVIGATION_TYPE_OTHER");
 		/* Filter domains here */
 		/* If the value of “mouse-button” is not 0, then the navigation was triggered by a mouse event.
 		 * test for link clicked but no button ? */
 		url = webkit_uri_request_get_uri(
 			webkit_navigation_action_get_request(a));
-		g_debug("Trying to open url: %s", url);
+		REMMINA_PLUGIN_DEBUG("Trying to open url: %s", url);
 		webkit_web_view_load_uri(gpdata->webview, url);
 		break;
 	default:
 		break;
 	}
-	g_debug("WEBKIT_NAVIGATION_TYPE is %d", webkit_navigation_action_get_navigation_type(a));
+	REMMINA_PLUGIN_DEBUG("WEBKIT_NAVIGATION_TYPE is %d", webkit_navigation_action_get_navigation_type(a));
 
 	webkit_policy_decision_ignore(decision);
 }
-static gboolean
-remmina_plugin_www_decide_resource(WebKitPolicyDecision *decision, RemminaProtocolWidget *gp)
+gboolean remmina_plugin_www_decide_resource(WebKitPolicyDecision *decision, RemminaProtocolWidget *gp)
 {
 	TRACE_CALL(__func__);
-	g_debug("Policy decision resource");
+	REMMINA_PLUGIN_DEBUG("Policy decision resource");
 	WebKitResponsePolicyDecision *response_decision =
 		WEBKIT_RESPONSE_POLICY_DECISION(decision);
 	WebKitURIResponse *response =
@@ -251,9 +308,9 @@ remmina_plugin_www_decide_resource(WebKitPolicyDecision *decision, RemminaProtoc
 
 	mime_type = webkit_uri_response_get_mime_type(response);
 
-	g_debug("Mymetype is %s", mime_type);
+	REMMINA_PLUGIN_DEBUG("The media type is %s", mime_type);
 
-	/* If WebKit can't handle the mime type start the download
+	/* If WebKit can't handle the media type, start the download
 	 * process */
 	if (webkit_response_policy_decision_is_mime_type_supported(response_decision))
 		return FALSE;
@@ -275,9 +332,9 @@ remmina_plugin_www_decide_resource(WebKitPolicyDecision *decision, RemminaProtoc
 	else if (!strncmp(mime_type, "application/octet-stream", 6))
 		type = WWW_WEB_VIEW_DOCUMENT_OCTET_STREAM;
 
-	g_debug("Document type is %i", type);
+	REMMINA_PLUGIN_DEBUG("Document type is %i", type);
 
-	/* FIXME: maybe it makes more sense to have an API to query the mime
+	/* FIXME: Maybe it makes more sense to have an API to query the media
 	 * type when the load of a page starts than doing this here.
 	 */
 	if (gpdata->document_type != type) {
@@ -294,14 +351,12 @@ static void remmina_www_web_view_js_finished(GObject *object, GAsyncResult *resu
 {
 	TRACE_CALL(__func__);
 
-	RemminaProtocolWidget *gp = (RemminaProtocolWidget *)user_data;
-	RemminaPluginWWWData *gpdata = GET_PLUGIN_DATA(gp);
 	WebKitJavascriptResult *js_result;
 	GError *error = NULL;
 
 	js_result = webkit_web_view_run_javascript_finish(WEBKIT_WEB_VIEW(object), result, &error);
 	if (!js_result) {
-		g_warning("Error running javascript: %s", error->message);
+		g_warning("Could not run JavaScript code: %s", error->message);
 		g_error_free(error);
 		return;
 	}
@@ -315,18 +370,17 @@ static void remmina_www_web_view_js_finished(GObject *object, GAsyncResult *resu
 		str_value = jsc_value_to_string(value);
 		exception = jsc_context_get_exception(jsc_value_get_context(value));
 		if (exception)
-			g_warning("Error running javascript: %s", jsc_exception_get_message(exception));
+			g_warning("Could not run JavaScript code: %s", jsc_exception_get_message(exception));
 		else
 			g_print("Script result: %s\n", str_value);
 		g_free(str_value);
 	} else {
 		str_value = jsc_value_to_string(value);
-		g_warning("javascript: unexpected return value, is not a string: %s", str_value);
+		g_warning("Received something other than a string from JavaScript: %s", str_value);
 		g_free(str_value);
 	}
 #endif
 	if (js_result) webkit_javascript_result_unref(js_result);
-	gpdata->formauthenticated = TRUE;
 }
 
 static gboolean remmina_www_query_feature(RemminaProtocolWidget *gp, const RemminaProtocolFeature *feature)
@@ -335,13 +389,16 @@ static gboolean remmina_www_query_feature(RemminaProtocolWidget *gp, const Remmi
 	return TRUE;
 }
 
+static gboolean remmina_plugin_www_load_failed_tls_cb(WebKitWebView *webview,
+						      gchar *failing_uri, GTlsCertificate *certificate,
+						      GTlsCertificateFlags errors, RemminaProtocolWidget *gp) __attribute__ ((unused));
 static gboolean remmina_plugin_www_load_failed_tls_cb(WebKitWebView *webview,
 						      gchar *failing_uri, GTlsCertificate *certificate,
 						      GTlsCertificateFlags errors, RemminaProtocolWidget *gp)
 {
 	TRACE_CALL(__func__);
-	/* Avoid to fail if certificate is not good. TODO: Add widgets to let the user decide */
-	g_debug("Ignoring certificate and return TRUE");
+	/* Avoid failing if certificate is not good. TODO: Add widgets to let the user decide */
+	REMMINA_PLUGIN_DEBUG("Ignoring certificate and return TRUE");
 	return TRUE;
 }
 
@@ -370,7 +427,7 @@ static void remmina_plugin_www_init(RemminaProtocolWidget *gp)
 				PLUGIN_NAME,
 				NULL);
 	cache_dir = g_build_path("/", datapath, "cache", NULL);
-	g_debug("WWW data path is %s", datapath);
+	REMMINA_PLUGIN_DEBUG("WWW data path is %s", datapath);
 
 	if (datapath) {
 		gchar *indexeddb_dir = g_build_filename(datapath, "indexeddb", NULL);
@@ -407,11 +464,21 @@ static void remmina_plugin_www_init(RemminaProtocolWidget *gp)
 	/* enable-fullscreen, default TRUE, TODO: Try FALSE */
 
 #ifdef DEBUG
-	/* Enable the developer extras */
+	/* Turn on the developer extras */
 	webkit_settings_set_enable_developer_extras(gpdata->settings, TRUE);
+	webkit_settings_set_enable_write_console_messages_to_stdout(gpdata->settings, TRUE);
 #endif
 
-	/* user-agent. TODO: Add option. */
+	/* allow-file-access-from-file-urls */
+	webkit_settings_set_allow_file_access_from_file_urls(gpdata->settings, TRUE);
+	/* allow-modal-dialogs */
+	webkit_settings_set_allow_modal_dialogs(gpdata->settings, TRUE);
+	/* enable-caret-browsing */
+	webkit_settings_set_enable_caret_browsing(gpdata->settings, TRUE);
+	/* enable-html5-database */
+	webkit_settings_set_enable_html5_database(gpdata->settings, TRUE);
+
+	/* user-agent. */
 	if (remmina_plugin_service->file_get_string(remminafile, "user-agent")) {
 		gchar *useragent = g_strdup(remmina_plugin_service->file_get_string(remminafile, "user-agent"));
 		webkit_settings_set_user_agent(gpdata->settings, useragent);
@@ -446,17 +513,46 @@ static void remmina_plugin_www_init(RemminaProtocolWidget *gp)
 	/* enable-webgl */
 	if (remmina_plugin_service->file_get_int(remminafile, "enable-webgl", FALSE)) {
 		webkit_settings_set_enable_webgl(gpdata->settings, TRUE);
+		webkit_settings_set_enable_accelerated_2d_canvas(gpdata->settings, TRUE);
 		g_info("enable-webgl enabled");
 	}
 
 	if (remmina_plugin_service->file_get_int(remminafile, "ignore-tls-errors", FALSE)) {
-		webkit_web_context_set_tls_errors_policy(gpdata->context, WEBKIT_TLS_ERRORS_POLICY_IGNORE);
+#if WEBKIT_CHECK_VERSION(2, 32, 0)
+		webkit_website_data_manager_set_tls_errors_policy(
+			gpdata->data_mgr, WEBKIT_TLS_ERRORS_POLICY_IGNORE);
+#else
+		webkit_web_context_set_tls_errors_policy(
+			gpdata->context, WEBKIT_TLS_ERRORS_POLICY_IGNORE);
+#endif
 		g_info("Ignore TLS errors");
 	}
+	if (remmina_plugin_service->file_get_string(remminafile, "proxy-url")) {
+		gchar *proxyurl = g_strdup(remmina_plugin_service->file_get_string(remminafile, "proxy-url"));
+		WebKitNetworkProxySettings *proxy_settings = webkit_network_proxy_settings_new (proxyurl, NULL);
+#if WEBKIT_CHECK_VERSION(2, 32, 0)
+		webkit_website_data_manager_set_network_proxy_settings(
+			gpdata->data_mgr, WEBKIT_NETWORK_PROXY_MODE_CUSTOM, proxy_settings);
+#else
+		webkit_web_context_set_network_proxy_settings(
+				gpdata->context, WEBKIT_NETWORK_PROXY_MODE_CUSTOM,proxy_settings);
+#endif
+		webkit_network_proxy_settings_free(proxy_settings);
+		g_free(proxyurl);
+	}
 
 	webkit_web_context_set_automation_allowed(gpdata->context, TRUE);
 	webkit_settings_set_javascript_can_open_windows_automatically(gpdata->settings, TRUE);
 	webkit_settings_set_allow_modal_dialogs(gpdata->settings, TRUE);
+	/** Frames flattening
+	 * Some websites engage in embedding frames-inside-of-frames. WebKit has
+	 * the ability to flatten them so they behave, when scrolling, as one big
+	 * frame. If for some reason it is not enabled, go ahead and turn it on.
+	 */
+	if (!webkit_settings_get_enable_frame_flattening(gpdata->settings)) {
+		webkit_settings_set_enable_frame_flattening(gpdata->settings, true);
+	}
+	webkit_settings_set_enable_resizable_text_areas(gpdata->settings, true);
 
 	g_signal_connect(G_OBJECT(gpdata->context), "download-started",
 			 G_CALLBACK(remmina_plugin_www_download_started), gp);
@@ -473,7 +569,6 @@ static gboolean remmina_plugin_www_on_auth(WebKitWebView *webview, WebKitAuthent
 	gboolean disablepasswordstoring;
 	RemminaFile *remminafile;
 
-
 	gpdata = (RemminaPluginWWWData *)g_object_get_data(G_OBJECT(gp), "plugin-data");
 
 	g_info("Authenticate");
@@ -481,13 +576,25 @@ static gboolean remmina_plugin_www_on_auth(WebKitWebView *webview, WebKitAuthent
 	remminafile = remmina_plugin_service->protocol_plugin_get_file(gp);
 
 	disablepasswordstoring = remmina_plugin_service->file_get_int(remminafile, "disablepasswordstoring", FALSE);
-	ret = remmina_plugin_service->protocol_plugin_init_authuserpwd(gp, FALSE, !disablepasswordstoring);
-
+	ret = remmina_plugin_service->protocol_plugin_init_auth(gp,
+		(disablepasswordstoring ? 0 : REMMINA_MESSAGE_PANEL_FLAG_SAVEPASSWORD) | REMMINA_MESSAGE_PANEL_FLAG_USERNAME,
+		_("Enter WWW authentication credentials"),
+		remmina_plugin_service->file_get_string(remminafile, "username"),
+		remmina_plugin_service->file_get_string(remminafile, "password"),
+		NULL,
+		NULL);
 	if (ret == GTK_RESPONSE_OK) {
 		s_username = remmina_plugin_service->protocol_plugin_init_get_username(gp);
 		s_password = remmina_plugin_service->protocol_plugin_init_get_password(gp);
-		if (remmina_plugin_service->protocol_plugin_init_get_savepassword(gp))
+
+		save = remmina_plugin_service->protocol_plugin_init_get_savepassword(gp);
+		if (save) {
+			remmina_plugin_service->file_set_string(remminafile, "username", s_username);
 			remmina_plugin_service->file_set_string(remminafile, "password", s_password);
+		} else {
+			remmina_plugin_service->file_set_string(remminafile, "username", NULL);
+			remmina_plugin_service->file_set_string(remminafile, "password", NULL);
+		}
 		if (request) {
 			gpdata->credentials = webkit_credential_new(
 				g_strdup(s_username),
@@ -496,22 +603,8 @@ static gboolean remmina_plugin_www_on_auth(WebKitWebView *webview, WebKitAuthent
 			webkit_authentication_request_authenticate(request, gpdata->credentials);
 			webkit_credential_free(gpdata->credentials);
 		}
-
-		save = remmina_plugin_service->protocol_plugin_init_get_savepassword(gp);
-		if (save) {
-			// User has requested to save credentials. We put all the new credentials
-			// into remminafile->settings. They will be saved later, on successful connection, by
-			// rcw.c
-
-			remmina_plugin_service->file_set_string(remminafile, "username", s_username);
-			remmina_plugin_service->file_set_string(remminafile, "password", s_password);
-		}
-
 		if (s_username) g_free(s_username);
 		if (s_password) g_free(s_password);
-
-		/* Free credentials */
-
 		gpdata->authenticated = TRUE;
 	} else {
 		gpdata->authenticated = FALSE;
@@ -538,7 +631,7 @@ static void remmina_plugin_www_form_auth(WebKitWebView *webview,
 
 	remminafile = remmina_plugin_service->protocol_plugin_get_file(gp);
 
-	g_debug("load-changed emitted");
+	REMMINA_PLUGIN_DEBUG("load-changed emitted");
 
 
 	const gchar *const *dirs = g_get_system_data_dirs();
@@ -565,8 +658,10 @@ static void remmina_plugin_www_form_auth(WebKitWebView *webview,
 
 	switch (load_event) {
 	case WEBKIT_LOAD_STARTED:
+		REMMINA_PLUGIN_DEBUG("Load started");
 		break;
 	case WEBKIT_LOAD_REDIRECTED:
+		REMMINA_PLUGIN_DEBUG("Load redirected");
 		break;
 	case WEBKIT_LOAD_COMMITTED:
 		/* The load is being performed. Current URI is
@@ -574,38 +669,50 @@ static void remmina_plugin_www_form_auth(WebKitWebView *webview,
 		 * load is requested or a navigation within the
 		 * same page is performed
 		 * uri = webkit_web_view_get_uri (webview); */
+		REMMINA_PLUGIN_DEBUG("Load committed");
 		break;
 	case WEBKIT_LOAD_FINISHED:
 		/* Load finished, we can now set user/password
-		 * in the html form */
-		g_debug("Load finished");
+		 * in the HTML form */
+		REMMINA_PLUGIN_DEBUG("Load finished");
 		if (gpdata && gpdata->formauthenticated == TRUE)
 			break;
 
 		if (remmina_plugin_service->file_get_string(remminafile, "username") ||
 		    remmina_plugin_service->file_get_string(remminafile, "password")) {
-			g_debug("Authentication is enabled");
-			g_file_get_contents(www_js_file, &s_js, NULL, NULL);
-			jsstr = g_string_new(s_js);
-			if (remmina_plugin_service->file_get_string(remminafile, "username"))
-				www_utils_string_replace_all(jsstr, "USRPLACEHOLDER",
-							     remmina_plugin_service->file_get_string(remminafile, "username"));
-			if (remmina_plugin_service->file_get_string(remminafile, "password"))
-				www_utils_string_replace_all(jsstr, "PWDPLACEHOLDER",
-							     remmina_plugin_service->file_get_string(remminafile, "password"));
-			s_js = g_string_free(jsstr, FALSE);
+			REMMINA_PLUGIN_DEBUG("Authentication is enabled");
+			if (www_js_file) {
+				error = NULL;
+				if (g_file_get_contents(www_js_file, &s_js, NULL, &error)) {
+					jsstr = g_string_new(s_js);
+					if (remmina_plugin_service->file_get_string(remminafile, "username"))
+						www_utils_string_replace_all(jsstr, "USRPLACEHOLDER",
+										remmina_plugin_service->file_get_string(remminafile, "username"));
+					if (remmina_plugin_service->file_get_string(remminafile, "password"))
+						www_utils_string_replace_all(jsstr, "PWDPLACEHOLDER",
+										remmina_plugin_service->file_get_string(remminafile, "password"));
+					s_js = g_string_free(jsstr, FALSE);
 
-			if (!s_js || s_js[0] == '\0') {
-				break;
-			} else {
-				g_debug("We are trying to send this JS: %s", s_js);
-				webkit_web_view_run_javascript(
-						webview,
-						s_js,
-						NULL,
-						remmina_www_web_view_js_finished,
-						gp);
-				g_free(s_js);
+					if (!s_js || s_js[0] == '\0') {
+						break;
+					} else {
+						REMMINA_PLUGIN_DEBUG("Trying to send this JavaScript: %s", s_js);
+						webkit_web_view_run_javascript(
+								webview,
+								s_js,
+								NULL,
+								remmina_www_web_view_js_finished,
+								gp);
+						g_free(s_js);
+					}
+				} else {
+					if (error) {
+						REMMINA_PLUGIN_DEBUG("Unable to read file: %s\n", error->message);
+						g_error_free(error);
+					} else {
+						REMMINA_PLUGIN_DEBUG("Unable to read file. No error returned from glib.\n");
+					}
+				}
 			}
 		}
 		break;
@@ -618,6 +725,7 @@ static gboolean remmina_plugin_www_close_connection(RemminaProtocolWidget *gp)
 	RemminaPluginWWWData *gpdata;
 	gpdata = (RemminaPluginWWWData *)g_object_get_data(G_OBJECT(gp), "plugin-data");
 
+	webkit_web_view_stop_loading(gpdata->webview);
 	webkit_web_view_try_close(gpdata->webview);
 
 	if (gpdata->url) g_free(gpdata->url);
@@ -631,7 +739,7 @@ static gboolean remmina_plugin_www_close_connection(RemminaProtocolWidget *gp)
 	/* Remove instance->context from gp object data to avoid double free */
 	g_object_steal_data(G_OBJECT(gp), "plugin-data");
 
-	remmina_plugin_service->protocol_plugin_emit_signal(gp, "disconnect");
+	remmina_plugin_service->protocol_plugin_signal_connection_closed(gp);
 	return FALSE;
 }
 
@@ -656,13 +764,14 @@ static gboolean remmina_plugin_www_open_connection(RemminaProtocolWidget *gp)
 
 	if (remmina_plugin_service->file_get_string(remminafile, "username") ||
 	    remmina_plugin_service->file_get_string(remminafile, "password")) {
-		g_debug("Authentication is enabled");
+		REMMINA_PLUGIN_DEBUG("Authentication is enabled");
 		remmina_plugin_www_on_auth(gpdata->webview, NULL, gp);
 	}
 
 	//"signal::load-failed-with-tls-errors", G_CALLBACK(remmina_plugin_www_load_failed_tls_cb), gp,
 	g_object_connect(
 		G_OBJECT(gpdata->webview),
+		"signal::create", G_CALLBACK(remmina_plugin_www_on_create), gp,
 		"signal::load-changed", G_CALLBACK(remmina_plugin_www_form_auth), gp,
 		"signal::authenticate", G_CALLBACK(remmina_plugin_www_on_auth), gp,
 		"signal::decide-policy", G_CALLBACK(remmina_plugin_www_decide_policy_cb), gp,
@@ -680,7 +789,7 @@ static gboolean remmina_plugin_www_open_connection(RemminaProtocolWidget *gp)
 		webkit_web_inspector_show(WEBKIT_WEB_INSPECTOR(inspector));
 	}
 #endif
-	remmina_plugin_service->protocol_plugin_emit_signal(gp, "connect");
+	remmina_plugin_service->protocol_plugin_signal_connection_opened(gp);
 	gtk_widget_show_all(gpdata->box);
 
 	return TRUE;
@@ -717,14 +826,7 @@ static void remmina_plugin_www_save_snapshot(GObject *object, GAsyncResult *resu
 
 	screenshot = gdk_pixbuf_get_from_surface(surface, 0, 0, width, height);
 	if (screenshot == NULL)
-		g_debug("WWW: gdk_pixbuf_get_from_surface failed");
-
-	// Transfer the PixBuf in the main clipboard selection
-	gchar *value = remmina_plugin_service->pref_get_value("deny_screenshot_clipboard");
-	if (value && value == FALSE) {
-		GtkClipboard *c = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
-		gtk_clipboard_set_image(c, screenshot);
-	}
+		REMMINA_PLUGIN_DEBUG("WWW: gdk_pixbuf_get_from_surface failed");
 
 	pngstr = g_string_new(g_strdup_printf("%s/%s.png",
 					      remmina_plugin_service->pref_get_value("screenshot_path"),
@@ -746,7 +848,7 @@ static void remmina_plugin_www_save_snapshot(GObject *object, GAsyncResult *resu
 				     g_strdup_printf("%f", g_date_time_get_seconds(date)));
 	g_date_time_unref(date);
 	pngname = g_string_free(pngstr, FALSE);
-	g_debug("Saving screenshot as %s", pngname);
+	REMMINA_PLUGIN_DEBUG("Saving screenshot as %s", pngname);
 
 	cairo_surface_write_to_png(surface, pngname);
 	if (g_file_test(pngname, G_FILE_TEST_EXISTS))
@@ -776,14 +878,22 @@ static gboolean remmina_plugin_www_get_snapshot(RemminaProtocolWidget *gp, Remmi
  * c) Setting description
  * d) Compact disposition
  * e) Values for REMMINA_PROTOCOL_SETTING_TYPE_SELECT or REMMINA_PROTOCOL_SETTING_TYPE_COMBO
- * f) Unused pointer
+ * f) Setting tooltip
+ * g) Validation data pointer, will be passed to the validation callback method.
+ * h) Validation callback method (Can be NULL. Every entry will be valid then.)
+ *		use following prototype:
+ *		gboolean mysetting_validator_method(gpointer key, gpointer value,
+ *						    gpointer validator_data);
+ *		gpointer key is a gchar* containing the setting's name,
+ *		gpointer value contains the value which should be validated,
+ *		gpointer validator_data contains your passed data.
  */
 static const RemminaProtocolSetting remmina_plugin_www_basic_settings[] =
 {
-	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	  "server",   N_("URL (http://address or https://address)"), FALSE, NULL, NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	  "username", N_("Username"),				     FALSE, NULL, NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD, "password", N_("Password"),				     FALSE, NULL, NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_END,	  NULL,	      NULL,					     FALSE, NULL, NULL }
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	  "server",   N_("URL"),	FALSE, NULL, N_("http://address or https://address"),	NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	  "username", N_("Username"),   FALSE, NULL, NULL,					NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD, "password", N_("Password"),   FALSE, NULL, NULL,					NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_END,	  NULL,	      NULL,		FALSE, NULL, NULL,					NULL, NULL }
 };
 
 /* Array of RemminaProtocolSetting for advanced settings.
@@ -793,21 +903,22 @@ static const RemminaProtocolSetting remmina_plugin_www_basic_settings[] =
  * c) Setting description
  * d) Compact disposition
  * e) Values for REMMINA_PROTOCOL_SETTING_TYPE_SELECT or REMMINA_PROTOCOL_SETTING_TYPE_COMBO
- * f) Unused pointer
+ * f) Setting Tooltip
  */
 static const RemminaProtocolSetting remmina_plugin_www_advanced_settings[] =
 {
-	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,  "user-agent",		    N_("User Agent"),		       FALSE, NULL, NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "enable-java",		    N_("Enable Java support"),	       TRUE,  NULL, NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "enable-smooth-scrolling",   N_("Enable smooth scrolling"),     TRUE,  NULL, NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "enable-spatial-navigation", N_("Enable Spatial Navigation"),   TRUE,  NULL, NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "enable-plugins",	    N_("Enable support for plugins"),  TRUE,  NULL, NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "enable-webgl",		    N_("Enable support for WebGL"),    TRUE,  NULL, NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "enable-webaudio",	    N_("Enable support for WebAudio"), TRUE,  NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,  "user-agent",		    N_("User agent"),		       FALSE, NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,  "proxy-url",		    N_("Proxy URL"),		       FALSE, NULL, N_("E.g. https://example.org, socks://mysocks:1080") },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "enable-java",		    N_("Turn on Java support"),	       TRUE,  NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "enable-smooth-scrolling",   N_("Turn on smooth scrolling"),     TRUE,  NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "enable-spatial-navigation", N_("Turn on spatial navigation"),   TRUE,  NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "enable-plugins",	    N_("Turn on plugin support"),  TRUE,  NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "enable-webgl",		    N_("Turn on WebGL support"),    TRUE,  NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "enable-webaudio",	    N_("Turn on HTML5 audio support"), TRUE,  NULL, NULL },
 	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "ignore-tls-errors",	    N_("Ignore TLS errors"),	       TRUE,  NULL, NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disablepasswordstoring",    N_("Disable password storing"),    TRUE,  NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disablepasswordstoring",    N_("Forget passwords after use"),    TRUE,  NULL, NULL },
 #ifdef DEBUG
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "enable-webinspector",	    N_("Enable Web Inspector"),	       TRUE,  NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "enable-webinspector",	    N_("Turn on Web Inspector"),	       TRUE,  NULL, NULL },
 #endif
 	{ REMMINA_PROTOCOL_SETTING_TYPE_END,   NULL,			    NULL,			       FALSE, NULL, NULL }
 };
@@ -837,9 +948,11 @@ static RemminaProtocolPlugin remmina_plugin =
 	remmina_plugin_www_open_connection,     // Plugin open connection
 	remmina_plugin_www_close_connection,    // Plugin close connection
 	remmina_www_query_feature,              // Query for available features
-	NULL,                                   // Call a feature
-	NULL,                                   // Send a keystroke
-	remmina_plugin_www_get_snapshot         // Capture screenshot
+	NULL,                                   // Call feature
+	NULL,                                   // Send keystroke
+	remmina_plugin_www_get_snapshot,        // Capture screenshot
+	NULL,                                   // RCW map event
+	NULL                                    // RCW unmap event
 };
 
 G_MODULE_EXPORT gboolean remmina_plugin_entry(RemminaPluginService *service)
