@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2009-2011 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2020 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2021 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -410,6 +410,7 @@ remmina_file_load(const gchar *filename)
 			upgrade_sshkeys_202001(remminafile);
 
 		}
+		g_strfreev(keys);
 	} else {
 		REMMINA_DEBUG ("Unable to load remmina profile file %s: cannot find key name= in section remmina.\n", filename);
 		remminafile = NULL;
@@ -434,7 +435,7 @@ void remmina_file_set_string_ref(RemminaFile *remminafile, const gchar *setting,
 	if (value) {
 		/* We refuse to accept to set the "resolution" field */
 		if (strcmp(setting, "resolution") == 0) {
-			message = "WARNING: the \"resolution\" setting in .pref files is deprecated, but some code in remmina or in a plugin is trying to set it.\n";
+			message = "WARNING: the “resolution” setting in .pref files is deprecated, but some code in remmina or in a plugin is trying to set it.\n";
 			fputs(message, stdout);
 			remmina_main_show_warning_dialog(message);
 			return;
@@ -470,7 +471,7 @@ remmina_file_get_string(RemminaFile *remminafile, const gchar *setting)
 	}
 
 	if (strcmp(setting, "resolution") == 0) {
-		message = "WARNING: the \"resolution\" setting in .pref files is deprecated, but some code in remmina or in a plugin is trying to read it.\n";
+		message = "WARNING: the “resolution” setting in .pref files is deprecated, but some code in remmina or in a plugin is trying to read it.\n";
 		fputs(message, stdout);
 		remmina_main_show_warning_dialog(message);
 		return NULL;
@@ -487,7 +488,7 @@ remmina_file_get_secret(RemminaFile *remminafile, const gchar *setting)
 
 	/* This function is in the RemminaPluginService table, we cannot remove it
 	 * without breaking plugin API */
-	g_warning("remmina_file_get_secret(remminafile,\"%s\") is deprecated and must not be called. Use remmina_file_get_string() and do not deallocate returned memory.\n", setting);
+	g_warning("remmina_file_get_secret(remminafile,“%s”) is deprecated and must not be called. Use remmina_file_get_string() and do not deallocate returned memory.\n", setting);
 	return g_strdup(remmina_file_get_string(remminafile, setting));
 }
 
@@ -507,7 +508,7 @@ gchar *remmina_file_format_properties(RemminaFile *remminafile, const gchar *set
 	remmina_utils_string_replace_all(fmt_str, "%g", remmina_file_get_string(remminafile, "group"));
 
 	now = g_date_time_new_now_local ();
-	date_str = g_date_time_format(now, "YYYY-MM-DDTHH-MM-SSZ");
+	date_str = g_date_time_format(now, "%FT%TZ");
 	remmina_utils_string_replace_all(fmt_str, "%d", date_str);
 	g_free(date_str);
 
@@ -518,16 +519,48 @@ gchar *remmina_file_format_properties(RemminaFile *remminafile, const gchar *set
 void remmina_file_set_int(RemminaFile *remminafile, const gchar *setting, gint value)
 {
 	TRACE_CALL(__func__);
-	g_hash_table_insert(remminafile->settings, g_strdup(setting), g_strdup_printf("%i", value));
+	if (remminafile)
+		g_hash_table_insert(remminafile->settings,
+							g_strdup(setting),
+							g_strdup_printf("%i", value));
 }
 
 gint remmina_file_get_int(RemminaFile *remminafile, const gchar *setting, gint default_value)
 {
 	TRACE_CALL(__func__);
 	gchar *value;
+	gint r;
 
 	value = g_hash_table_lookup(remminafile->settings, setting);
-	return value == NULL ? default_value : (value[0] == 't' ? TRUE : atoi(value));
+	r = value == NULL ? default_value : (value[0] == 't' ? TRUE : atoi(value));
+	// TOO verbose: REMMINA_DEBUG ("Integer value is: %d", r);
+	return r;
+}
+
+// sscanf uses the set language to convert the float.
+// therefore '.' and ',' cannot be used interchangeably.
+gdouble remmina_file_get_double(RemminaFile *remminafile,
+							   const gchar *setting,
+							   gdouble default_value)
+{
+	TRACE_CALL(__func__);
+	gchar *value;
+
+	value = g_hash_table_lookup(remminafile->settings, setting);
+	if (!value)
+		return default_value;
+
+	// str to double.
+	// https://stackoverflow.com/questions/10075294/converting-string-to-a-double-variable-in-c
+	gdouble d;
+	gint ret = sscanf(value, "%lf", &d);
+	if (ret != 1) {
+		// failed.
+		d = default_value;
+	}
+
+	// TOO VERBOSE: REMMINA_DEBUG("Double value is: %lf", d);
+	return d;
 }
 
 static GKeyFile *
@@ -551,9 +584,13 @@ void remmina_file_free(RemminaFile *remminafile)
 	if (remminafile == NULL)
 		return;
 
-	g_free(remminafile->filename);
-	g_hash_table_destroy(remminafile->settings);
-	g_hash_table_destroy(remminafile->spsettings);
+	if (remminafile->filename)
+		g_free(remminafile->filename);
+	if (remminafile->settings)
+		g_hash_table_destroy(remminafile->settings);
+	if (remminafile->spsettings)
+		g_hash_table_destroy(remminafile->spsettings);
+
 	g_free(remminafile);
 }
 
@@ -752,7 +789,7 @@ void remmina_file_unsave_passwords(RemminaFile *remminafile)
 			setting_iter = protocol_plugin->basic_settings;
 			if (setting_iter) {
 				while (setting_iter->type != REMMINA_PROTOCOL_SETTING_TYPE_END) {
-					g_debug("setting name: %s", setting_iter->name);
+					// TOO VERBOSE: g_debug("setting name: %s", setting_iter->name);
 					if (setting_iter->name == NULL) {
 						g_error("Internal error: a setting name in protocol plugin %s is null. Please fix RemminaProtocolSetting struct content.", proto);
 					} else {
