@@ -339,7 +339,7 @@ void remmina_icon_init(void)
 	gboolean sni_supported;
 	char msg[200];
 
-	g_stpcpy(remmina_panel, "remmina-status");
+	g_stpcpy(remmina_panel, "org.remmina.Remmina-status");
 
 	/* Print on stdout the availability of appindicators on DBUS */
 	sni_supported = remmina_sysinfo_is_appindicator_available();
