@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2010-2011 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2019 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2021 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,6 +39,7 @@
 
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
+#include <gmodule.h>
 #include <gio/gio.h>
 #include <string.h>
 
@@ -52,11 +53,14 @@
 #include "remmina_widget_pool.h"
 #include "rcw.h"
 #include "remmina_plugin_manager.h"
+#include "remmina_plugin_native.h"
+#ifdef WITH_PYTHONLIBS
+#include "remmina_plugin_python.h"
+#endif
 #include "remmina_public.h"
 #include "remmina_masterthread_exec.h"
 #include "remmina/remmina_trace_calls.h"
 
-
 static GPtrArray* remmina_plugin_table = NULL;
 
 /* A GHashTable of GHashTables where to store the names of the encrypted settings */
@@ -101,8 +105,14 @@ static void init_settings_cache(RemminaPlugin *plugin)
 	}
 
 	/* Some settings are encrypted "by name" */
+	/* g_hash_table_insert(pht, g_strdup("proxy_password"), (gpointer)TRUE); */
+
+	g_hash_table_insert(pht, g_strdup("ssh_tunnel_password"), (gpointer)TRUE);
+	g_hash_table_insert(pht, g_strdup("ssh_tunnel_passphrase"), (gpointer)TRUE);
+
+	/* ssh_password is no longer used starting from remmina 1.4.
+	 * But we MUST mark it as encrypted setting, or the migration procedure will fail */
 	g_hash_table_insert(pht, g_strdup("ssh_password"), (gpointer)TRUE);
-	g_hash_table_insert(pht, g_strdup("ssh_passphrase"), (gpointer)TRUE);
 
 	/* Other settings are encrypted because of their type */
 	protocol_plugin = (RemminaProtocolPlugin *)plugin;
@@ -129,8 +139,8 @@ static gboolean remmina_plugin_manager_register_plugin(RemminaPlugin *plugin)
 {
 	TRACE_CALL(__func__);
 	if (plugin->type == REMMINA_PLUGIN_TYPE_SECRET) {
-		g_print("Remmina plugin %s (type=%s) has registered but not yet initialized/activated. "
-			"Initialization order is %d.\n", plugin->name,
+		g_print("Remmina plugin %s (type=%s) has been registered, but is not yet initialized/activated. "
+			"The initialization order is %d.\n", plugin->name,
 			_(remmina_plugin_type_name[plugin->type]),
 			((RemminaSecretPlugin*)plugin)->init_order);
 	}
@@ -160,8 +170,6 @@ gboolean remmina_gtksocket_available()
 	return available;
 }
 
-
-
 RemminaPluginService remmina_plugin_manager_service =
 {
 	remmina_plugin_manager_register_plugin,
@@ -182,9 +190,13 @@ RemminaPluginService remmina_plugin_manager_service =
 	remmina_protocol_widget_start_reverse_tunnel,
 	remmina_protocol_widget_start_xport_tunnel,
 	remmina_protocol_widget_set_display,
-	remmina_protocol_widget_close_connection,
-	remmina_protocol_widget_panel_authpwd,
-	remmina_protocol_widget_panel_authuserpwd,
+	remmina_protocol_widget_signal_connection_closed,
+	remmina_protocol_widget_signal_connection_opened,
+	remmina_protocol_widget_update_align,
+	remmina_protocol_widget_lock_dynres,
+	remmina_protocol_widget_unlock_dynres,
+	remmina_protocol_widget_desktop_resize,
+	remmina_protocol_widget_panel_auth,
 	remmina_protocol_widget_panel_new_certificate,
 	remmina_protocol_widget_panel_changed_certificate,
 	remmina_protocol_widget_get_username,
@@ -216,7 +228,7 @@ RemminaPluginService remmina_plugin_manager_service =
 	remmina_file_get_secret,
 	remmina_file_set_int,
 	remmina_file_get_int,
-	remmina_file_unsave_password,
+	remmina_file_unsave_passwords,
 
 	remmina_pref_set_value,
 	remmina_pref_get_value,
@@ -226,45 +238,48 @@ RemminaPluginService remmina_plugin_manager_service =
 	remmina_pref_get_ssh_parseconfig,
 	remmina_pref_keymap_get_keyval,
 
+	_remmina_info,
+	_remmina_message,
+	_remmina_debug,
+	_remmina_warning,
+	_remmina_error,
+	_remmina_critical,
 	remmina_log_print,
 	remmina_log_printf,
 
 	remmina_widget_pool_register,
 
 	rcw_open_from_file_full,
+	remmina_public_open_unix_sock,
 	remmina_public_get_server_port,
 	remmina_masterthread_exec_is_main_thread,
 	remmina_gtksocket_available,
 	remmina_protocol_widget_get_profile_remote_width,
 	remmina_protocol_widget_get_profile_remote_height
-
 };
 
+const char *get_filename_ext(const char *filename) {
+	const char* last = strrchr(filename, '/');
+    const char *dot = strrchr(last, '.');
+    if(!dot || dot == filename) return "";
+    return dot + 1;
+}
+
 static void remmina_plugin_manager_load_plugin(const gchar *name)
 {
-	TRACE_CALL(__func__);
-	GModule *module;
-	RemminaPluginEntryFunc entry;
+	const char* ext = get_filename_ext(name);
 
-	module = g_module_open(name, G_MODULE_BIND_LAZY | G_MODULE_BIND_LOCAL);
-
-	if (!module) {
-		g_print("Failed to load plugin: %s.\n", name);
-		g_print("Error: %s\n", g_module_error());
-		return;
+	if (g_str_equal(G_MODULE_SUFFIX, ext)) {
+		remmina_plugin_native_load(&remmina_plugin_manager_service, name);
+	} else if (g_str_equal("py", ext)) {
+#ifdef WITH_PYTHONLIBS
+		remmina_plugin_python_load(&remmina_plugin_manager_service, name);
+#else
+		REMMINA_DEBUG("Python support not compiled, cannot load Python plugins");
+#endif
+	} else {
+		g_print("%s: Skip unsupported file type '%s'\n", name, ext);
 	}
-
-	if (!g_module_symbol(module, "remmina_plugin_entry", (gpointer*)&entry)) {
-		g_print("Failed to locate plugin entry: %s.\n", name);
-		return;
-	}
-
-	if (!entry(&remmina_plugin_manager_service)) {
-		g_print("Plugin entry returned false: %s.\n", name);
-		return;
-	}
-
-	/* We don’t close the module because we will need it throughout the process lifetime */
 }
 
 static gint compare_secret_plugin_init_order(gconstpointer a, gconstpointer b)
@@ -281,7 +296,7 @@ static gint compare_secret_plugin_init_order(gconstpointer a, gconstpointer b)
 	return 0;
 }
 
-void remmina_plugin_manager_init(void)
+void remmina_plugin_manager_init()
 {
 	TRACE_CALL(__func__);
 	GDir *dir;
@@ -294,20 +309,25 @@ void remmina_plugin_manager_init(void)
 	GSList *sple;
 
 	remmina_plugin_table = g_ptr_array_new();
+#ifdef WITH_PYTHONLIBS
+	remmina_plugin_python_init();
+#endif
 
 	if (!g_module_supported()) {
-		g_print("Dynamic loading of plugins is not supported in this platform!\n");
+		g_print("Dynamic loading of plugins is not supported on this platform!\n");
 		return;
 	}
 
+	g_print("Load modules from %s\n", REMMINA_RUNTIME_PLUGINDIR);
 	dir = g_dir_open(REMMINA_RUNTIME_PLUGINDIR, 0, NULL);
+
 	if (dir == NULL)
 		return;
 	while ((name = g_dir_read_name(dir)) != NULL) {
 		if ((ptr = strrchr(name, '.')) == NULL)
 			continue;
 		ptr++;
-		if (g_strcmp0(ptr, G_MODULE_SUFFIX) != 0)
+		if (!remmina_plugin_manager_loader_supported(ptr))
 			continue;
 		fullpath = g_strdup_printf(REMMINA_RUNTIME_PLUGINDIR "/%s", name);
 		remmina_plugin_manager_load_plugin(fullpath);
@@ -334,7 +354,7 @@ void remmina_plugin_manager_init(void)
 	while(sple != NULL) {
 		sp = (RemminaSecretPlugin*)sple->data;
 		if (sp->init()) {
-			g_print("Secret plugin %s has been successfully initialized and will be your default secret plugin\n",
+			g_print("The %s secret plugin has been initialized and it will be your default secret plugin\n",
 				sp->name);
 			remmina_secret_plugin = sp;
 			break;
@@ -345,6 +365,11 @@ void remmina_plugin_manager_init(void)
 	g_slist_free(secret_plugins);
 }
 
+gboolean remmina_plugin_manager_loader_supported(const char *filetype) {
+	TRACE_CALL(__func__);
+	return g_str_equal("py", filetype) || g_str_equal(G_MODULE_SUFFIX, filetype);
+}
+
 RemminaPlugin* remmina_plugin_manager_get_plugin(RemminaPluginType type, const gchar *name)
 {
 	TRACE_CALL(__func__);
@@ -554,7 +579,3 @@ gboolean remmina_plugin_manager_is_encrypted_setting(RemminaProtocolPlugin *pp,
 
 	return TRUE;
 }
-
-
-
-
