@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2009-2011 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2020 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2021 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,6 +37,7 @@
 #include "config.h"
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
+#include <errno.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -67,6 +68,9 @@ remmina_public_create_combo_entry(const gchar *text, const gchar *def, gboolean
 	gchar *buf, *ptr1, *ptr2;
 	gint i;
 
+	//g_debug("text: %s\n", text);
+	//g_debug("def: %s\n", def);
+
 	combo = gtk_combo_box_text_new_with_entry();
 	found = FALSE;
 
@@ -330,7 +334,39 @@ remmina_public_combine_path(const gchar *path1, const gchar *path2)
 	return g_strdup_printf("%s/%s", path1, path2);
 }
 
-void remmina_public_get_server_port(const gchar *server, gint defaultport, gchar **host, gint *port)
+//static int remmina_public_open_unix_sock(const char *unixsock, GError **error)
+gint remmina_public_open_unix_sock(const char *unixsock)
+{
+    struct sockaddr_un addr;
+    int fd;
+
+    if (strlen(unixsock) + 1 > sizeof(addr.sun_path)) {
+        //g_set_error(error, REMMINA_ERROR, REMMINA_ERROR_FAILED,
+                    g_debug(_("Address is too long for UNIX socket_path: %s"), unixsock);
+        return -1;
+    }
+
+    memset(&addr, 0, sizeof addr);
+    addr.sun_family = AF_UNIX;
+    strcpy(addr.sun_path, unixsock);
+
+    if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0) {
+        //g_set_error(error, REMMINA_ERROR, REMMINA_ERROR_FAILED,
+                    g_debug(_("Creating UNIX socket failed: %s"), g_strerror(errno));
+        return -1;
+    }
+
+    if (connect(fd, (struct sockaddr *)&addr, sizeof addr) < 0) {
+        //g_set_error(error, REMMINA_ERROR, REMMINA_ERROR_FAILED,
+                    g_debug(_("Connecting to UNIX socket failed: %s"), g_strerror(errno));
+        close(fd);
+        return -1;
+    }
+
+    return fd;
+}
+
+void remmina_public_get_server_port_old(const gchar *server, gint defaultport, gchar **host, gint *port)
 {
 	TRACE_CALL(__func__);
 	gchar *str, *ptr, *ptr2;
@@ -353,6 +389,8 @@ void remmina_public_get_server_port(const gchar *server, gint defaultport, gchar
 			if (port)
 				*port = defaultport;
 			g_free(str);
+			g_debug ("(%s) - host: %s", __func__, *host);
+			g_debug ("(%s) - port: %d", __func__, *port);
 			return;
 		}
 
@@ -374,6 +412,50 @@ void remmina_public_get_server_port(const gchar *server, gint defaultport, gchar
 		g_free(str);
 	if (port)
 		*port = defaultport;
+
+	g_debug ("(%s) - host: %s", __func__, *host);
+	g_debug ("(%s) - port: %d", __func__, *port);
+}
+
+void remmina_public_get_server_port(const gchar *server, gint defaultport, gchar **host, gint *port)
+{
+	TRACE_CALL(__func__);
+
+	const gchar *nul_terminated_server = NULL;
+	if (server != NULL) {
+		if(strstr(g_strdup(server), "ID:") != NULL) {
+			g_debug ("(%s) - Using remmina_public_get_server_port_old to parse the repeater ID", __func__);
+			remmina_public_get_server_port_old (server, defaultport, host, port);
+			return;
+		}
+
+		GNetworkAddress *address;
+		GError *err = NULL;
+
+		nul_terminated_server = g_strdup (server);
+		g_debug ("(%s) - Parsing server: %s, default port: %d", __func__, server, defaultport);
+		address = (GNetworkAddress*)g_network_address_parse ((const gchar *)nul_terminated_server, defaultport, &err);
+
+		if (address == NULL) {
+			g_debug ("(%s) - Error converting server string: %s, with error: %s", __func__, nul_terminated_server, err->message);
+			*host = NULL;
+			if (err)
+				g_error_free(err);
+		} else {
+
+			*host = g_strdup(g_network_address_get_hostname (address));
+			*port = g_network_address_get_port (address);
+		}
+	} else
+		*host = NULL;
+
+	if (port == 0)
+		*port = defaultport;
+
+	g_debug ("(%s) - host: %s", __func__, *host);
+	g_debug ("(%s) - port: %d", __func__, *port);
+
+	return;
 }
 
 gboolean remmina_public_get_xauth_cookie(const gchar *display, gchar **msg)
@@ -440,111 +522,6 @@ gint remmina_public_open_xdisplay(const gchar *disp)
 	return sock;
 }
 
-/* This function was copied from GEdit (gedit-utils.c). */
-guint remmina_public_get_current_workspace(GdkScreen *screen)
-{
-	TRACE_CALL(__func__);
-#ifdef GDK_WINDOWING_X11
-#if GTK_CHECK_VERSION(3, 10, 0)
-	g_return_val_if_fail(GDK_IS_SCREEN(screen), 0);
-	if (GDK_IS_X11_DISPLAY(gdk_screen_get_display(screen)))
-		return gdk_x11_screen_get_current_desktop(screen);
-	else
-		return 0;
-
-#else
-	GdkWindow *root_win;
-	GdkDisplay *display;
-	Atom type;
-	gint format;
-	gulong nitems;
-	gulong bytes_after;
-	guint *current_desktop;
-	gint err, result;
-	guint ret = 0;
-
-	g_return_val_if_fail(GDK_IS_SCREEN(screen), 0);
-
-	root_win = gdk_screen_get_root_window(screen);
-	display = gdk_screen_get_display(screen);
-
-	gdk_error_trap_push();
-	result = XGetWindowProperty(GDK_DISPLAY_XDISPLAY(display), GDK_WINDOW_XID(root_win),
-		gdk_x11_get_xatom_by_name_for_display(display, "_NET_CURRENT_DESKTOP"),
-		0, G_MAXLONG, False, XA_CARDINAL, &type, &format, &nitems,
-		&bytes_after, (gpointer) & current_desktop);
-	err = gdk_error_trap_pop();
-
-	if (err != Success || result != Success)
-		return ret;
-
-	if (type == XA_CARDINAL && format == 32 && nitems > 0)
-		ret = current_desktop[0];
-
-	XFree(current_desktop);
-	return ret;
-#endif
-#else
-	/* FIXME: on mac etc proably there are native APIs
-	 * to get the current workspace etc */
-	return 0;
-#endif
-}
-
-/* This function was copied from GEdit (gedit-utils.c). */
-guint remmina_public_get_window_workspace(GtkWindow *gtkwindow)
-{
-	TRACE_CALL(__func__);
-#ifdef GDK_WINDOWING_X11
-#if GTK_CHECK_VERSION(3, 10, 0)
-	GdkWindow *window;
-	g_return_val_if_fail(GTK_IS_WINDOW(gtkwindow), 0);
-	g_return_val_if_fail(gtk_widget_get_realized(GTK_WIDGET(gtkwindow)), 0);
-	window = gtk_widget_get_window(GTK_WIDGET(gtkwindow));
-	if (GDK_IS_X11_DISPLAY(gdk_window_get_display(window)))
-		return gdk_x11_window_get_desktop(window);
-	else
-		return 0;
-#else
-	GdkWindow *window;
-	GdkDisplay *display;
-	Atom type;
-	gint format;
-	gulong nitems;
-	gulong bytes_after;
-	guint *workspace;
-	gint err, result;
-	guint ret = 0;
-
-	g_return_val_if_fail(GTK_IS_WINDOW(gtkwindow), 0);
-	g_return_val_if_fail(gtk_widget_get_realized(GTK_WIDGET(gtkwindow)), 0);
-
-	window = gtk_widget_get_window(GTK_WIDGET(gtkwindow));
-	display = gdk_window_get_display(window);
-
-	gdk_error_trap_push();
-	result = XGetWindowProperty(GDK_DISPLAY_XDISPLAY(display), GDK_WINDOW_XID(window),
-		gdk_x11_get_xatom_by_name_for_display(display, "_NET_WM_DESKTOP"),
-		0, G_MAXLONG, False, XA_CARDINAL, &type, &format, &nitems,
-		&bytes_after, (gpointer) & workspace);
-	err = gdk_error_trap_pop();
-
-	if (err != Success || result != Success)
-		return ret;
-
-	if (type == XA_CARDINAL && format == 32 && nitems > 0)
-		ret = workspace[0];
-
-	XFree(workspace);
-	return ret;
-#endif
-#else
-	/* FIXME: on mac etc proably there are native APIs
-	 * to get the current workspace etc */
-	return 0;
-#endif
-}
-
 /* Find hardware keycode for the requested keyval */
 guint16 remmina_public_get_keycode_for_keyval(GdkKeymap *keymap, guint keyval)
 {
@@ -588,6 +565,21 @@ GtkBuilder* remmina_public_gtk_builder_new_from_file(gchar *filename)
 	return builder;
 }
 
+/* Load a GtkBuilder object from a resource */
+GtkBuilder* remmina_public_gtk_builder_new_from_resource(gchar *resource)
+{
+	TRACE_CALL(__func__);
+	GError *err = NULL;
+	GtkBuilder *builder = gtk_builder_new();
+	gtk_builder_add_from_resource (builder, resource, &err);
+	//GtkBuilder *builder = gtk_builder_new_from_resource (resource);
+	if (err != NULL) {
+		g_print("Error adding build from resource. Error: %s", err->message);
+		g_error_free(err);
+	}
+	return builder;
+}
+
 /* Change parent container for a widget
  * If possible use this function instead of the deprecated gtk_widget_reparent */
 void remmina_public_gtk_widget_reparent(GtkWidget *widget, GtkContainer *container)
@@ -646,13 +638,24 @@ void remmina_public_send_notification(const gchar *notification_id,
 {
 	TRACE_CALL(__func__);
 
-	GNotification *notification = g_notification_new(notification_title);
-	g_notification_set_body(notification, notification_message);
+	g_autoptr(GNotification) n = NULL;
+	gint priority = G_NOTIFICATION_PRIORITY_NORMAL;
+
+	n = g_notification_new(notification_title);
+	g_notification_set_body(n, notification_message);
+	if (g_strcmp0 (notification_id, "remmina-security-trust-all-id") == 0) {
+		g_debug ("remmina_public_send_notification: We got a remmina-security-trust-all-id notification");
+		priority = G_NOTIFICATION_PRIORITY_HIGH;
+		/** parameter 5 is the tab index for the security tab in the preferences
+		 * TODO: Do not hardcode the parameter
+		 * TODO: Do not hardcode implement DBus interface correctly of this won't work*/
+		g_notification_set_default_action_and_target (n, "app.preferences", "i", 5);
+		g_notification_add_button_with_target (n, _("Change security settings"), "app.preferences", "i", 5);
+	}
 #if GLIB_CHECK_VERSION(2, 42, 0)
-	g_notification_set_priority(notification, G_NOTIFICATION_PRIORITY_NORMAL);
+	g_notification_set_priority(n, priority);
 #endif
-	g_application_send_notification(g_application_get_default(), notification_id, notification);
-	g_object_unref(notification);
+	g_application_send_notification(g_application_get_default(), notification_id, n);
 }
 
 /* Replaces all occurrences of search in a new copy of string by replacement. */
@@ -683,8 +686,8 @@ gchar* remmina_public_str_replace_in_place(gchar *string, const gchar *search, c
 {
 	TRACE_CALL(__func__);
 	gchar *new_string = remmina_public_str_replace(string, search, replacement);
-	g_free(string);
 	string = g_strdup(new_string);
+	g_free(new_string);
 	return string;
 }
 
