@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2010 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2019 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2021 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,21 +48,38 @@
 #include "remmina_protocol_widget.h"
 #include "remmina_sftp_plugin.h"
 
-#define REMMINA_PLUGIN_SFTP_FEATURE_PREF_SHOW_HIDDEN   1
-#define REMMINA_PLUGIN_SFTP_FEATURE_PREF_OVERWRITE_ALL 2
+#define   REMMINA_PLUGIN_SFTP_FEATURE_PREF_SHOW_HIDDEN     1
+#define   REMMINA_PLUGIN_SFTP_FEATURE_PREF_OVERWRITE_ALL   2
+#define   REMMINA_PLUGIN_SFTP_FEATURE_PREF_RESUME_ALL      3
 
 #define REMMINA_PLUGIN_SFTP_FEATURE_PREF_OVERWRITE_ALL_KEY "overwrite_all"
+#define REMMINA_PLUGIN_SFTP_FEATURE_PREF_RESUME_ALL_KEY    "resume_all"
 
 #define GET_PLUGIN_DATA(gp) (RemminaPluginSftpData *)g_object_get_data(G_OBJECT(gp), "plugin-data");
 
 typedef struct _RemminaPluginSftpData {
-	GtkWidget *	client;
-	pthread_t	thread;
-	RemminaSFTP *	sftp;
+	RemminaSFTPClient *	client;
+	pthread_t		thread;
+	RemminaSFTP *		sftp;
 } RemminaPluginSftpData;
 
 static RemminaPluginService *remmina_plugin_service = NULL;
 
+gboolean remmina_plugin_sftp_start_direct_tunnel(RemminaProtocolWidget *gp, char **phost, int *pport)
+{
+	gchar *hostport;
+
+	hostport = remmina_plugin_service->protocol_plugin_start_direct_tunnel(gp, 22, FALSE);
+	if (hostport == NULL) {
+		remmina_plugin_service->protocol_plugin_signal_connection_closed(gp);
+		return FALSE;
+	}
+
+	remmina_plugin_service->get_server_port(hostport, 22, phost, pport);
+
+	return TRUE;
+}
+
 static gpointer
 remmina_plugin_sftp_main_thread(gpointer data)
 {
@@ -75,91 +92,62 @@ remmina_plugin_sftp_main_thread(gpointer data)
 	gboolean cont = FALSE;
 	gint ret;
 	const gchar *cs;
-	const gchar *saveserver;
-	const gchar *saveusername;
-	gchar *hostport;
-	gchar tunneluser[33];           /* On linux a username can have a 32 char lenght */
-	gchar tunnelserver[256];        /* On linux a servername can have a 255 char lenght */
-	gchar tunnelport[6];            /* A TCP port can have a maximum value of 65535 */
 	gchar *host;
-	gint port;
+	int port;
 
 	pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
 	CANCEL_ASYNC
 
 		gpdata = GET_PLUGIN_DATA(gp);
 
-	/* remmina_plugin_service->protocol_plugin_start_direct_tunnel start the
-	 * SSH Tunnel and return the server + port string
-	 * Therefore we set the SSH Tunnel username here, before the tunnel
-	 * is established and than set it back to the destination SSH user.
-	 *
-	 * */
 	remminafile = remmina_plugin_service->protocol_plugin_get_file(gp);
-	/* We save the SSH server name, so that we can restore it at the end of the connection */
-	saveserver = remmina_plugin_service->file_get_string(remminafile, "ssh_server");
-	remmina_plugin_service->file_set_string(remminafile, "save_ssh_server", g_strdup(saveserver));
-	/* We save the SSH username, so that we can restore it at the end of the connection */
-	saveusername = remmina_plugin_service->file_get_string(remminafile, "ssh_username");
-	remmina_plugin_service->file_set_string(remminafile, "save_ssh_username", g_strdup(saveusername));
 
-	if (saveserver) {
-		/* if the server string contains the character @ we extract the user
-		 * and the server string (host:port)
-		 * */
-		if (strchr(saveserver, '@')) {
-			sscanf(saveserver, "%[_a-zA-Z0-9.]@%[_a-zA-Z0-9.]:%[0-9]",
-			       tunneluser, tunnelserver, tunnelport);
-			g_print("Username: %s, tunneluser: %s\n",
-				remmina_plugin_service->file_get_string(remminafile, "ssh_username"), tunneluser);
-			if (saveusername != NULL && tunneluser[0] != '\0')
-				remmina_plugin_service->file_set_string(remminafile, "ssh_username", NULL);
-			remmina_plugin_service->file_set_string(remminafile, "ssh_username",
-								g_strdup(tunneluser));
-			remmina_plugin_service->file_set_string(remminafile, "ssh_server",
-								g_strconcat(tunnelserver, ":", tunnelport, NULL));
-		}
-	}
-
-	hostport = remmina_plugin_service->protocol_plugin_start_direct_tunnel(gp, 22, FALSE);
-	/* We restore the SSH username as the tunnel is set */
-	remmina_plugin_service->file_set_string(remminafile, "ssh_username", g_strdup(saveusername));
-	if (hostport == NULL)
-		return FALSE;
-	remmina_plugin_service->get_server_port(hostport, 22, &host, &port);
+	/* we may need to open a new tunnel too */
+	host = NULL;
+	port = 0;
+	if (!remmina_plugin_sftp_start_direct_tunnel(gp, &host, &port))
+		return NULL;
 
 	ssh = g_object_get_data(G_OBJECT(gp), "user-data");
 	if (ssh) {
 		/* Create SFTP connection based on existing SSH session */
 		sftp = remmina_sftp_new_from_ssh(ssh);
+		ssh->tunnel_entrance_host = host;
+		ssh->tunnel_entrance_port = port;
 		if (remmina_ssh_init_session(REMMINA_SSH(sftp)) &&
-		    remmina_ssh_auth(REMMINA_SSH(sftp), NULL, gp, remminafile) > 0 &&
+		    remmina_ssh_auth(REMMINA_SSH(sftp), NULL, gp, remminafile) == REMMINA_SSH_AUTH_SUCCESS &&
 		    remmina_sftp_open(sftp))
 			cont = TRUE;
 	} else {
 		/* New SFTP connection */
-		if (remmina_plugin_service->file_get_int(remminafile, "ssh_enabled", FALSE))
-			remmina_plugin_service->file_set_string(remminafile, "ssh_server", g_strdup(hostport));
-		else
-			remmina_plugin_service->file_set_string(remminafile, "ssh_server",
-								remmina_plugin_service->file_get_string(remminafile, "server"));
-		g_free(hostport);
-		g_free(host);
-
 		sftp = remmina_sftp_new_from_file(remminafile);
+		ssh = REMMINA_SSH(sftp);
+		ssh->tunnel_entrance_host = host;
+		ssh->tunnel_entrance_port = port;
 		while (1) {
-			if (!remmina_ssh_init_session(REMMINA_SSH(sftp))) {
-				remmina_plugin_service->protocol_plugin_set_error(gp, "%s", REMMINA_SSH(sftp)->error);
+			if (!remmina_ssh_init_session(ssh)) {
+				remmina_plugin_service->protocol_plugin_set_error(gp, "%s", ssh->error);
 				break;
 			}
 
-			ret = remmina_ssh_auth_gui(REMMINA_SSH(sftp), gp, remminafile);
-			if (ret == 0)
-				remmina_plugin_service->protocol_plugin_set_error(gp, "%s", REMMINA_SSH(sftp)->error);
-			if (ret <= 0) break;
+			ret = remmina_ssh_auth_gui(ssh, gp, remminafile);
+			if (ret != REMMINA_SSH_AUTH_SUCCESS) {
+				if (ret == REMMINA_SSH_AUTH_RECONNECT) {
+					if (ssh->session) {
+						ssh_disconnect(ssh->session);
+						ssh_free(ssh->session);
+						ssh->session = NULL;
+					}
+					g_free(ssh->callback);
+					continue;
+				}
+				if (ret != REMMINA_SSH_AUTH_USERCANCEL)
+					remmina_plugin_service->protocol_plugin_set_error(gp, "%s", ssh->error);
+				break;
+			}
 
 			if (!remmina_sftp_open(sftp)) {
-				remmina_plugin_service->protocol_plugin_set_error(gp, "%s", REMMINA_SSH(sftp)->error);
+				remmina_plugin_service->protocol_plugin_set_error(gp, "%s", ssh->error);
 				break;
 			}
 
@@ -170,14 +158,11 @@ remmina_plugin_sftp_main_thread(gpointer data)
 			cont = TRUE;
 			break;
 		}
-
-		/* We restore the ssh_server name */
-		remmina_plugin_service->file_set_string(remminafile, "ssh_server",
-							remmina_plugin_service->file_get_string(remminafile, "save_ssh_server"));
 	}
+
 	if (!cont) {
 		if (sftp) remmina_sftp_free(sftp);
-		IDLE_ADD((GSourceFunc)remmina_plugin_service->protocol_plugin_close_connection, gp);
+		remmina_plugin_service->protocol_plugin_signal_connection_closed(gp);
 		return NULL;
 	}
 
@@ -185,7 +170,7 @@ remmina_plugin_sftp_main_thread(gpointer data)
 	/* RemminaSFTPClient owns the object, we just take the reference */
 	gpdata->sftp = sftp;
 
-	remmina_plugin_service->protocol_plugin_emit_signal(gp, "connect");
+	remmina_plugin_service->protocol_plugin_signal_connection_opened(gp);
 
 	gpdata->thread = 0;
 	return NULL;
@@ -214,8 +199,9 @@ remmina_plugin_sftp_init(RemminaProtocolWidget *gp)
 	remminafile = remmina_plugin_service->protocol_plugin_get_file(gp);
 
 	gpdata->client = remmina_sftp_client_new();
-	gtk_widget_show(gpdata->client);
-	gtk_container_add(GTK_CONTAINER(gp), gpdata->client);
+	gpdata->client->gp = gp;
+	gtk_widget_show(GTK_WIDGET(gpdata->client));
+	gtk_container_add(GTK_CONTAINER(gp), GTK_WIDGET(gpdata->client));
 
 	remmina_ftp_client_set_show_hidden(REMMINA_FTP_CLIENT(gpdata->client),
 					   remmina_plugin_service->file_get_int(remminafile, "showhidden", FALSE));
@@ -223,8 +209,11 @@ remmina_plugin_sftp_init(RemminaProtocolWidget *gp)
 	remmina_ftp_client_set_overwrite_status(REMMINA_FTP_CLIENT(gpdata->client),
 						remmina_plugin_service->file_get_int(remminafile,
 										     REMMINA_PLUGIN_SFTP_FEATURE_PREF_OVERWRITE_ALL_KEY, FALSE));
+	remmina_ftp_client_set_resume_status(REMMINA_FTP_CLIENT(gpdata->client),
+					     remmina_plugin_service->file_get_int(remminafile,
+										  REMMINA_PLUGIN_SFTP_FEATURE_PREF_RESUME_ALL_KEY, FALSE));
 
-	remmina_plugin_service->protocol_plugin_register_hostkey(gp, gpdata->client);
+	remmina_plugin_service->protocol_plugin_register_hostkey(gp, GTK_WIDGET(gpdata->client));
 
 	g_signal_connect(G_OBJECT(gpdata->client),
 			 "realize", G_CALLBACK(remmina_plugin_sftp_client_on_realize), gp);
@@ -242,7 +231,7 @@ remmina_plugin_sftp_open_connection(RemminaProtocolWidget *gp)
 
 	if (pthread_create(&gpdata->thread, NULL, remmina_plugin_sftp_main_thread, gp)) {
 		remmina_plugin_service->protocol_plugin_set_error(gp,
-								  "Failed to initialize pthread. Falling back to non-thread mode…");
+								  "Could not initialize pthread. Falling back to non-thread mode…");
 		gpdata->thread = 0;
 		return FALSE;
 	} else {
@@ -265,7 +254,7 @@ remmina_plugin_sftp_close_connection(RemminaProtocolWidget *gp)
 	}
 
 	remmina_ftp_client_save_state(REMMINA_FTP_CLIENT(gpdata->client), remminafile);
-	remmina_plugin_service->protocol_plugin_emit_signal(gp, "disconnect");
+	remmina_plugin_service->protocol_plugin_signal_connection_closed(gp);
 	/* The session preference overwrite_all is always saved to FALSE in order
 	 * to avoid unwanted overwriting.
 	 * If we'd change idea just remove the next line to save the preference. */
@@ -308,6 +297,10 @@ remmina_plugin_sftp_call_feature(RemminaProtocolWidget *gp, const RemminaProtoco
 		remmina_ftp_client_set_overwrite_status(REMMINA_FTP_CLIENT(gpdata->client),
 							remmina_plugin_service->file_get_int(remminafile,
 											     REMMINA_PLUGIN_SFTP_FEATURE_PREF_OVERWRITE_ALL_KEY, FALSE));
+	case REMMINA_PLUGIN_SFTP_FEATURE_PREF_RESUME_ALL:
+		remmina_ftp_client_set_resume_status(REMMINA_FTP_CLIENT(gpdata->client),
+						     remmina_plugin_service->file_get_int(remminafile,
+											  REMMINA_PLUGIN_SFTP_FEATURE_PREF_RESUME_ALL_KEY, FALSE));
 		return;
 	}
 }
@@ -329,7 +322,10 @@ static const RemminaProtocolFeature remmina_plugin_sftp_features[] =
 	{ REMMINA_PROTOCOL_FEATURE_TYPE_PREF, REMMINA_PLUGIN_SFTP_FEATURE_PREF_SHOW_HIDDEN,   GINT_TO_POINTER(REMMINA_PROTOCOL_FEATURE_PREF_CHECK), "showhidden",
 	  N_("Show Hidden Files") },
 	{ REMMINA_PROTOCOL_FEATURE_TYPE_PREF, REMMINA_PLUGIN_SFTP_FEATURE_PREF_OVERWRITE_ALL, GINT_TO_POINTER(REMMINA_PROTOCOL_FEATURE_PREF_CHECK),
-	  REMMINA_PLUGIN_SFTP_FEATURE_PREF_OVERWRITE_ALL_KEY, N_("Overwrite all") },
+	  REMMINA_PLUGIN_SFTP_FEATURE_PREF_OVERWRITE_ALL_KEY, N_("Overwrite all files") },
+	{ REMMINA_PROTOCOL_FEATURE_TYPE_PREF, REMMINA_PLUGIN_SFTP_FEATURE_PREF_RESUME_ALL,    GINT_TO_POINTER(REMMINA_PROTOCOL_FEATURE_PREF_CHECK),
+	  REMMINA_PLUGIN_SFTP_FEATURE_PREF_RESUME_ALL_KEY, N_("Resume all file transfers") },
+	{ REMMINA_PROTOCOL_FEATURE_TYPE_TOOL, REMMINA_PROTOCOL_FEATURE_TOOL_SSH,	      N_("Connect via SSH from a new terminal"),	    "utilities-terminal",NULL},
 	{ REMMINA_PROTOCOL_FEATURE_TYPE_END,  0,					      NULL,						    NULL,
 	  NULL }
 };
@@ -341,17 +337,26 @@ static const RemminaProtocolFeature remmina_plugin_sftp_features[] =
  * c) Setting description
  * d) Compact disposition
  * e) Values for REMMINA_PROTOCOL_SETTING_TYPE_SELECT or REMMINA_PROTOCOL_SETTING_TYPE_COMBO
- * f) Unused pointer
+ * f) Setting tooltip
+ * g) Validation data pointer, will be passed to the validation callback method.
+ * h) Validation callback method (Can be NULL. Every entry will be valid then.)
+ *		use following prototype:
+ *		gboolean mysetting_validator_method(gpointer key, gpointer value,
+ *						    gpointer validator_data);
+ *		gpointer key is a gchar* containing the setting's name,
+ *		gpointer value contains the value which should be validated,
+ *		gpointer validator_data contains your passed data.
  */
 static const RemminaProtocolSetting remmina_sftp_basic_settings[] =
 {
-	{ REMMINA_PROTOCOL_SETTING_TYPE_SERVER,	  "ssh_server",	    NULL,			  FALSE, "_sftp-ssh._tcp", NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	  "ssh_username",   N_("User name"),		  FALSE, NULL,		   NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD, "ssh_password",   N_("User password"),	  FALSE, NULL,		   NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT,	  "ssh_auth",	    N_("Authentication type"),	  FALSE, ssh_auth,	   NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_FILE,	  "ssh_privatekey", N_("Identity file"),	  FALSE, NULL,		   NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD, "ssh_passphrase", N_("Private key passphrase"), FALSE, NULL,		   NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_END,	  NULL,		    NULL,			  FALSE, NULL,		   NULL }
+	{ REMMINA_PROTOCOL_SETTING_TYPE_SERVER,	  "server",	      NULL,				    FALSE, "_sftp-ssh._tcp", NULL, NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	  "username",	      N_("Username"),			    FALSE, NULL,	     NULL, NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD, "password",	      N_("Password"),			    FALSE, NULL,	     NULL, NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT,	  "ssh_auth",	      N_("Authentication type"),	    FALSE, ssh_auth,	     NULL, NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_FILE,	  "ssh_privatekey",   N_("SSH identity file"),		    FALSE, NULL,	     NULL, NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD, "ssh_passphrase",   N_("Password to unlock private key"), FALSE, NULL,	     NULL, NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	  "ssh_proxycommand", N_("SSH Proxy Command"),		    FALSE, NULL,	     NULL, NULL, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_END,	  NULL,		      NULL,				    FALSE, NULL,	     NULL, NULL, NULL }
 };
 
 /* Protocol plugin definition and features */
@@ -362,8 +367,8 @@ static RemminaProtocolPlugin remmina_plugin_sftp =
 	N_("SFTP - Secure File Transfer"),              // Description
 	GETTEXT_PACKAGE,                                // Translation domain
 	VERSION,                                        // Version number
-	"remmina-sftp-symbolic",                        // Icon for normal connection
-	"remmina-sftp-symbolic",                        // Icon for SSH connection
+	"org.remmina.Remmina-sftp-symbolic",                        // Icon for normal connection
+	"org.remmina.Remmina-sftp-symbolic",                        // Icon for SSH connection
 	remmina_sftp_basic_settings,                    // Array for basic settings
 	NULL,                                           // Array for advanced settings
 	REMMINA_PROTOCOL_SSH_SETTING_TUNNEL,            // SSH settings type
@@ -374,7 +379,9 @@ static RemminaProtocolPlugin remmina_plugin_sftp =
 	remmina_plugin_sftp_query_feature,              // Query for available features
 	remmina_plugin_sftp_call_feature,               // Call a feature
 	NULL,                                           // Send a keystroke
-	NULL                                            // No screenshot support available
+	NULL,                                           // Screenshot support unavailable
+	NULL,                                           // RCW map event
+	NULL                                            // RCW unmap event
 };
 
 void
